@@ -13,28 +13,41 @@
 #include "databaseinfo.h"
 #include "filter.h"
 #include "game.h"
-#include "pgndatabase.h"
 #include "memorydatabase.h"
+#include "pgndatabase.h"
+#include "polyglotdatabase.h"
 #include "settings.h"
 
-DatabaseInfo::DatabaseInfo()
+DatabaseInfo::DatabaseInfo(QUndoGroup* undoGroup)
 {
     m_database = new MemoryDatabase;
     m_filter = new Filter(m_database);
+    connect(m_filter, SIGNAL(searchProgress(int)), SIGNAL(searchProgress(int)));
+    connect(m_filter, SIGNAL(searchFinished()), SIGNAL(searchFinished()));
     m_bLoaded = true;
     m_utf8 = false;
-    m_undoStack = new QUndoStack(this);
+    m_undoStack = new QUndoStack((QObject*)undoGroup);
     newGame();
+    connect(m_undoStack, SIGNAL(cleanChanged(bool)), SLOT(dbCleanChanged(bool)));
+    connect(&m_game, SIGNAL(signalGameModified(bool,Game,QString)),SLOT(setModified(bool,Game,QString)));
+    connect(&m_game, SIGNAL(signalMoveChanged()), SIGNAL(signalMoveChanged()));
 }
 
-DatabaseInfo::DatabaseInfo(const QString& fname): m_filter(0), m_index(NewGame)
+DatabaseInfo::DatabaseInfo(QUndoGroup* undoGroup, const QString& fname): m_filter(0), m_index(NewGame)
 {
     m_filename = fname;
     m_bLoaded = false;
     m_utf8 = false;
-    m_undoStack = new QUndoStack(this);
+    m_undoStack = new QUndoStack((QObject*)undoGroup);
+    connect(m_undoStack, SIGNAL(cleanChanged(bool)), SLOT(dbCleanChanged(bool)));
+    connect(&m_game, SIGNAL(signalGameModified(bool,Game,QString)),SLOT(setModified(bool,Game,QString)));
+    connect(&m_game, SIGNAL(signalMoveChanged()), SIGNAL(signalMoveChanged()));
     QFile file(fname);
-    if(file.size() < 1024 * 1024 * AppSettings->getValue("/General/EditLimit").toInt())
+    if (IsPolyglotBook())
+    {
+        m_database = new PolyglotDatabase;
+    }
+    else if(file.size() < 1024 * 1024 * AppSettings->getValue("/General/EditLimit").toInt())
     {
         m_database = new MemoryDatabase;
     }
@@ -50,6 +63,7 @@ DatabaseInfo::DatabaseInfo(const QString& fname): m_filter(0), m_index(NewGame)
 
 void DatabaseInfo::doLoadFile(QString filename)
 {
+    if (m_filter) m_filter->cancel();
     if(!m_database->open(filename, m_utf8))
     {
         emit LoadFinished(this);
@@ -58,6 +72,8 @@ void DatabaseInfo::doLoadFile(QString filename)
     m_database->parseFile();
     delete m_filter;
     m_filter = new Filter(m_database);
+    connect(m_filter, SIGNAL(searchProgress(int)), SIGNAL(searchProgress(int)));
+    connect(m_filter, SIGNAL(searchFinished()), SIGNAL(searchFinished()));
     m_bLoaded = true;
     emit LoadFinished(this);
 }
@@ -71,6 +87,7 @@ void DatabaseInfo::run()
 
 bool DatabaseInfo::open(bool utf8)
 {
+    m_gameModified = false;
     m_bLoaded = false;
     m_utf8 = utf8;
     start();
@@ -79,6 +96,10 @@ bool DatabaseInfo::open(bool utf8)
 
 void DatabaseInfo::close()
 {
+    if (m_filter)
+    {
+        m_filter->cancel();
+    }
     m_bLoaded = false;
     m_database->m_break = true;
     if(isRunning())
@@ -89,14 +110,10 @@ void DatabaseInfo::close()
             terminate();
         }
     }
-    if(m_database)
-    {
-        delete m_database;
-    }
-    if(m_filter)
-    {
-        delete m_filter;
-    }
+
+    delete m_database;
+    delete m_filter;
+
     m_database = NULL;
     m_filter = NULL;
     m_undoStack->clear();
@@ -106,7 +123,7 @@ DatabaseInfo::~DatabaseInfo()
 {
 }
 
-bool DatabaseInfo::loadGame(int index, bool reload)
+bool DatabaseInfo::loadGame(int index)
 {
     if(!m_bLoaded)
     {
@@ -116,11 +133,7 @@ bool DatabaseInfo::loadGame(int index, bool reload)
     {
         return false;
     }
-    if(!reload && m_index == index)
-    {
-        return true;
-    }
-    if(!m_database || index < 0 || index >= m_database->count())
+    if(!m_database || index < 0 || index >= (int)m_database->count())
     {
         return false;
     }
@@ -128,7 +141,6 @@ bool DatabaseInfo::loadGame(int index, bool reload)
     {
         return false;
     }
-    m_undoStack->clear();
     m_index = index;
     int n = m_filter->gamePosition(index) - 1;
     if(n < 0)
@@ -136,17 +148,60 @@ bool DatabaseInfo::loadGame(int index, bool reload)
         n = 0;
     }
     m_game.moveToId(n);
-    m_game.setModified(false);
+    m_undoStack->clear();
+    setModified(false, Game(), "");
     return true;
 }
 
+void DatabaseInfo::dbCleanChanged(bool bClean)
+{
+    m_gameModified = !bClean;
+}
+
+bool DatabaseInfo::modified() const
+{
+    return m_gameModified;
+}
+
+bool DatabaseInfo::gameNeedsSaving() const
+{
+    return (isValid() && m_gameModified && !m_database->isReadOnly());
+}
+
+void DatabaseInfo::setModified(bool modified, const Game& g, QString action)
+{
+    if (modified)
+    {
+        if (!action.isEmpty())
+        {
+            m_undoStack->push(new GameUndoCommand(this, g, m_game, action));
+        }
+    }
+    else
+    {
+        m_undoStack->clear();
+    }
+    m_gameModified = modified;
+    emit signalGameModified();
+}
+
+QUndoStack *DatabaseInfo::undoStack() const
+{
+    return m_undoStack;
+}
+
+void DatabaseInfo::restoreState(const Game& game)
+{
+    emit signalRestoreState(game);
+}
+
 void DatabaseInfo::newGame()
 {
     m_undoStack->clear();
     m_game.clearTags();
     m_game.clear();
-    m_game.setModified(false);
     m_index = NewGame;
+    setModified(false, Game(), "");
 }
 
 bool DatabaseInfo::saveGame()
@@ -174,11 +229,12 @@ bool DatabaseInfo::saveGame()
         }
     }
 
-    if(m_index < m_database->count() && m_index >= 0)
+    if(m_index < (int)m_database->count() && m_index >= 0)
     {
         if(m_database->replace(m_index, m_game))
         {
-            m_game.setModified(false);
+            setModified(false, Game(), "");
+            m_undoStack->setClean();
             return true;
         }
     }
@@ -190,7 +246,8 @@ bool DatabaseInfo::saveGame()
         {
             database()->index()->setTag("ECO", eco, m_index);
         }
-        m_game.setModified(false);
+        setModified(false, Game(), "");
+        m_undoStack->setClean();
         return true;
     }
     return false;
@@ -200,8 +257,35 @@ void DatabaseInfo::resetFilter()
 {
     if(m_filter)
     {
+        m_filter->cancel();
         m_filter->resize(m_database->count());
         m_filter->setAll(1);
     }
 }
 
+bool DatabaseInfo::IsLoaded() const
+{
+    return m_bLoaded;
+}
+
+bool DatabaseInfo::IsUtf8() const
+{
+    return m_utf8;
+}
+
+bool DatabaseInfo::IsPGN() const
+{
+    if (m_filename.isEmpty())
+    {
+        return true;
+    }
+    QFileInfo fi(m_filename);
+    return (fi.suffix() == "pgn");
+}
+
+bool DatabaseInfo::IsPolyglotBook() const
+{
+    QFileInfo fi(m_filename);
+    return (fi.suffix() == "bin");
+}
+
