@@ -8,9 +8,13 @@
  *   (at your option) any later version.                                   *
  ***************************************************************************/
 
-#include "database.h"
 #include "filtermodel.h"
 
+#include "common.h"
+#include "database.h"
+#include "filter.h"
+#include "game.h"
+
 #include <QtGui>
 
 FilterModel::FilterModel(Filter* filter, QObject* parent)
@@ -18,9 +22,9 @@ FilterModel::FilterModel(Filter* filter, QObject* parent)
 {
     m_columnNames << tr("Nr")
                   << tr("White")
-                  << tr("WhiteElo")
+                  << tr("White Elo")
                   << tr("Black")
-                  << tr("BlackElo")
+                  << tr("Black Elo")
                   << tr("Event")
                   << tr("Site")
                   << tr("Round")
@@ -190,9 +194,3 @@ Filter* FilterModel::filter()
 {
     return m_filter;
 }
-
-void FilterModel::sort(int /*column*/, Qt::SortOrder /*order*/)
-{
-
-}
-
