@@ -12,6 +12,12 @@
 #include <QFile>
 #include "game.h"
 
+const QRegExp Game::emt("\\[%emt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+const QRegExp Game::clk("\\[%clk\\s*(\\d:\\d\\d:\\d\\d)\\]");
+const QRegExp Game::egt("\\[%egt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+const QRegExp Game::csl("\\[%csl\\s*([^\\]]*)\\]");
+const QRegExp Game::cal("\\[%cal\\s*([^\\]]*)\\]");
+
 QMap<quint64, QString> Game::m_ecoPositions;
 
 Game::Game() : QObject()
@@ -36,8 +42,6 @@ Game& Game::operator=(const Game & game)
         m_startingBoard = game.m_startingBoard;
         m_variationStartAnnotations = game.m_variationStartAnnotations;
         m_annotations = game.m_annotations;
-        m_squareAnnotations = game.m_squareAnnotations;
-        m_arrowAnnotations = game.m_arrowAnnotations;
         m_currentNode = game.m_currentNode;
         m_startPly = game.m_startPly;
         m_currentBoard = game.m_currentBoard;
@@ -51,6 +55,12 @@ Game::~Game()
 {
 }
 
+void Game::copyFromGame(const Game& g)
+{
+    *this = g;
+    emit signalGameModified(true,*this,"");
+}
+
 MoveId Game::dbAddMove(const Move& move, const QString& annotation, NagSet nags)
 {
     MoveNode node;
@@ -247,9 +257,8 @@ MoveId Game::findMergePoint(const Game& otherGame)
     return otherMergeNode;
 }
 
-void Game::mergeWithGame(const Game& g)
+void Game::dbMergeWithGame(const Game& g)
 {
-    Game state = *this;
     MoveId saveNode = m_currentNode;
     Game otherGame = g;
 
@@ -264,8 +273,6 @@ void Game::mergeWithGame(const Game& g)
 
     MoveId otherMergeNode = findMergePoint(otherGame);
 
-    // todo
-
     if(otherMergeNode != NO_MOVE)
     {
         backward();
@@ -290,7 +297,12 @@ void Game::mergeWithGame(const Game& g)
         moveToId(saveNode);
     }
     compact();
+}
 
+void Game::mergeWithGame(const Game& g)
+{
+    Game state = *this;
+    dbMergeWithGame(g);
     emit signalGameModified(true,state,tr("Merge game"));
 }
 
@@ -549,13 +561,8 @@ MoveId Game::dbAddVariation(const QString& sanMove, const QString& annotation, N
     return (m_moveNodes.size() - 1);
 }
 
-bool Game::promoteVariation(MoveId variation)
+void Game::dbPromoteVariation(MoveId variation)
 {
-    if(isMainline(variation))
-    {
-        return false;
-    }
-    Game state = *this;
     MoveId currentNode = m_currentNode;	// Save current move
 
     // Find first move of the variation
@@ -575,6 +582,16 @@ bool Game::promoteVariation(MoveId variation)
     qSwap(m_moveNodes[parent].nextNode, m_moveNodes[parent].variations[index]);
 
     moveToId(currentNode);	// Restore current move
+}
+
+bool Game::promoteVariation(MoveId variation)
+{
+    if(isMainline(variation))
+    {
+        return false;
+    }
+    Game state = *this;
+    dbPromoteVariation(variation);
     emit signalGameModified(true, state, tr("Promote Variation"));
     return true;
 }
@@ -631,6 +648,13 @@ void Game::truncateVariation(Position position)
     if(position == AfterMove)
     {
         MoveId node = m_moveNodes[m_currentNode].nextNode;
+        if (isMainline(node))
+        {
+            foreach(MoveId var, m_moveNodes[m_currentNode].variations)
+            {
+                removeNode(var);
+            }
+        }
         removeNode(node);
     }
     else if(position == BeforeMove && m_currentNode != 0)
@@ -718,8 +742,6 @@ void Game::removeComments()
     Game state = *this;
     m_variationStartAnnotations.clear();
     m_annotations.clear();
-    m_squareAnnotations.clear();
-    m_arrowAnnotations.clear();
     for(int i = 0; i < m_moveNodes.size(); ++i)
     {
         m_moveNodes[i].nags.clear();
@@ -840,6 +862,7 @@ bool Game::setAnnotation(QString annotation, MoveId moveId, Position position)
     Game state = *this;
     if (dbSetAnnotation(annotation, moveId, position))
     {
+        indicateAnnotationsOnBoard(currentMove());
         emit signalGameModified(true, state, tr("Set annotation"));
         return true;
     }
@@ -854,29 +877,6 @@ bool Game::dbSetAnnotation(QString annotation, MoveId moveId, Position position)
         return false;
     }
 
-    QString specAnnot = specialAnnotation(annotation, "[%csl");
-    if(position == Game::AfterMove || node == 0)
-    {
-        setSquareAnnotation(specAnnot, node);
-    }
-    else
-    {
-        setSquareAnnotation(specAnnot, m_moveNodes[node].previousNode);
-    }
-
-    specAnnot = specialAnnotation(annotation, "[%cal");
-    if(position == Game::AfterMove || node == 0)
-    {
-        setArrowAnnotation(specAnnot, node);
-    }
-    else
-    {
-        setArrowAnnotation(specAnnot, m_moveNodes[node].previousNode);
-    }
-
-    annotation.remove(QRegExp("\\[%csl[^\\]]*\\]"));
-    annotation.remove(QRegExp("\\[%cal[^\\]]*\\]"));
-
     int moves;
     int comments;
     int nags;
@@ -922,14 +922,14 @@ bool Game::setSquareAnnotation(QString squareAnnotation, MoveId moveId)
         return false;
     }
 
-    if(squareAnnotation.isEmpty())
+    QString s = annotation(moveId);
+    s.remove(csl);
+
+    if(!squareAnnotation.isEmpty())
     {
-        m_squareAnnotations.remove(node);
-    }
-    else
-    {
-        m_squareAnnotations[node] = squareAnnotation;
+        s.append(QString("[%csl %1]").arg(squareAnnotation));
     }
+    dbSetAnnotation(s, moveId);
     return true;
 }
 
@@ -977,7 +977,7 @@ bool Game::appendSquareAnnotation(Square s, QChar colorCode)
             newAnnot.truncate(newAnnot.length() - 1);
         }
     }
-    setSquareAnnotation(newAnnot);
+    setSquareAnnotation(newAnnot.trimmed());
     indicateAnnotationsOnBoard(currentMove());
     emit signalGameModified(true, state, tr("Colorize square"));
     return true;
@@ -1033,18 +1033,8 @@ bool Game::appendArrowAnnotation(Square dest, Square src, QChar colorCode)
 
 QString Game::squareAnnotation(MoveId moveId) const
 {
-    MoveId node = nodeValid(moveId);
-    if(node == NO_MOVE)
-    {
-        return "";
-    }
-
-    QString annotation = m_squareAnnotations[node];
-    if(annotation.isNull())
-    {
-        return "";
-    }
-    return m_squareAnnotations[node].simplified();
+    QString s = specAnnotation(csl, moveId);
+    return s;
 }
 
 bool Game::setArrowAnnotation(QString arrowAnnotation, MoveId moveId)
@@ -1055,97 +1045,24 @@ bool Game::setArrowAnnotation(QString arrowAnnotation, MoveId moveId)
         return false;
     }
 
-    if(arrowAnnotation.isEmpty())
+    QString s = annotation(moveId);
+    s.remove(cal);
+
+    if(!arrowAnnotation.isEmpty())
     {
-        m_arrowAnnotations.remove(node);
-    }
-    else
-    {
-        m_arrowAnnotations[node] = arrowAnnotation;
+        s.append(QString("[%cal %1]").arg(arrowAnnotation));
     }
+    dbSetAnnotation(s, moveId);
     return true;
 }
 
 QString Game::arrowAnnotation(MoveId moveId) const
 {
-    MoveId node = nodeValid(moveId);
-    if(node == NO_MOVE)
-    {
-        return "";
-    }
-
-    QString annotation = m_arrowAnnotations[node];
-    if(annotation.isNull())
-    {
-        return "";
-    }
-    return m_arrowAnnotations[node].simplified();
-}
-
-QString Game::egtAnnotation(MoveId moveId) const
-{
-    MoveId node = nodeValid(moveId);
-    if(node == NO_MOVE)
-    {
-        return "";
-    }
-
-    QString annotation = m_annotations[node];
-    if(annotation.isNull())
-    {
-        return "";
-    }
-
-    QRegExp egt("\\[%egt\\s*(\\d:\\d\\d:\\d\\d)\\]");
-    int pos = egt.indexIn(annotation);
-    if(pos >= 0)
-    {
-        return egt.cap(1);
-    }
-
-    return "";
-}
-
-QString Game::emtAnnotation(MoveId moveId) const
-{
-    MoveId node = nodeValid(moveId);
-    if(node == NO_MOVE)
-    {
-        return "";
-    }
-
-    QString annotation = m_annotations[node];
-    if(annotation.isNull())
-    {
-        return "";
-    }
-
-    QRegExp emt("\\[%emt\\s*(\\d:\\d\\d:\\d\\d)\\]");
-    int pos = emt.indexIn(annotation);
-    if(pos >= 0)
-    {
-        return emt.cap(1);
-    }
-
-    return "";
-}
-
-
-QString Game::timeAnnotation(MoveId moveId) const
-{
-    QString s = clkAnnotation(moveId);
-    if(s.isEmpty())
-    {
-        s = egtAnnotation(moveId);
-    }
-    if (s.isEmpty())
-    {
-        s = emtAnnotation(moveId);
-    }
+    QString s = specAnnotation(cal, moveId);
     return s;
 }
 
-QString Game::clkAnnotation(MoveId moveId) const
+QString Game::specAnnotation(const QRegExp& r, MoveId moveId) const
 {
     MoveId node = nodeValid(moveId);
     if(node == NO_MOVE)
@@ -1159,46 +1076,40 @@ QString Game::clkAnnotation(MoveId moveId) const
         return "";
     }
 
-    QRegExp clk("\\[%clk\\s*(\\d:\\d\\d:\\d\\d)\\]");
-    int pos = clk.indexIn(annotation);
+    int pos = r.indexIn(annotation);
     if(pos >= 0)
     {
-        return clk.cap(1);
+        return r.cap(1);
     }
 
     return "";
 }
 
-QString Game::composeAnnotation(QString freeText, MoveId node) const
+QString Game::timeAnnotation(MoveId moveId) const
 {
-    if(!m_squareAnnotations[node].isEmpty())
+    QString s = specAnnotation(clk, moveId);
+    if(s.isEmpty())
     {
-        freeText.append(QString("[%csl %1]").arg(m_squareAnnotations[node]));
+        s = specAnnotation(egt, moveId);
     }
-    if(!m_arrowAnnotations[node].isEmpty())
+    if (s.isEmpty())
     {
-        freeText.append(QString("[%cal %1]").arg(m_arrowAnnotations[node]));
+        s = specAnnotation(emt, moveId);
     }
-    return freeText;
+    return s;
 }
 
 QString Game::annotation(MoveId moveId, Position position) const
 {
     MoveId node = nodeValid(moveId);
-    if(position == AfterMove)
+
+    if ((position == AfterMove) || (node == 0))
     {
-        return composeAnnotation(m_annotations.value(node, ""), node);
+        return m_annotations.value(node, "");
     }
     else
     {
-        if(node == 0)
-        {
-            return composeAnnotation(m_annotations.value(0, ""), 0);
-        }
-        else
-        {
-            return m_variationStartAnnotations.value(node, "");
-        }
+        return m_variationStartAnnotations.value(node, "");
     }
 }
 
@@ -1206,27 +1117,22 @@ QString Game::textAnnotation(MoveId moveId, Position position) const
 {
     QString s;
     MoveId node = nodeValid(moveId);
-    if(position == AfterMove)
+
+    if ((position == AfterMove) || (node == 0))
     {
         s = m_annotations.value(node, "");
     }
     else
     {
-        if(node == 0)
-        {
-            s = m_annotations.value(0, "");
-        }
-        else
-        {
-            s = m_variationStartAnnotations.value(node, "");
-        }
+        s = m_variationStartAnnotations.value(node, "");
     }
-    QRegExp emt("\\[%emt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+
     s.remove(emt);
-    QRegExp egt("\\[%egt\\s*(\\d:\\d\\d:\\d\\d)\\]");
     s.remove(egt);
-    QRegExp clk("\\[%clk\\s*(\\d:\\d\\d:\\d\\d)\\]");
     s.remove(clk);
+    s.remove(csl);
+    s.remove(cal);
+
     return s;
 }
 
@@ -1239,7 +1145,7 @@ bool Game::canHaveStartAnnotation(MoveId moveId) const
 bool Game::addNag(Nag nag, MoveId moveId)
 {
     MoveId node = nodeValid(moveId);
-    if(node != NO_MOVE)
+    if ((node != NO_MOVE) && (nag != NullNag))
     {
         Game state = *this;
         m_moveNodes[node].nags.addNag(nag);
@@ -1593,21 +1499,24 @@ void Game::moveToId(MoveId moveId)
         return;
     }
 
-    //jump to node, travelling back to start adding the moves to the stack
-    MoveId node = moveId;
-    QStack < Move > moveStack;
-    while(node)
+    if (m_currentNode != moveId)
     {
-        moveStack.push(m_moveNodes[node].move);
-        node = m_moveNodes[node].previousNode;
-    }
+        //jump to node, travelling back to start adding the moves to the stack
+        MoveId node = moveId;
+        QStack < Move > moveStack;
+        while(node)
+        {
+            moveStack.push(m_moveNodes[node].move);
+            node = m_moveNodes[node].previousNode;
+        }
 
-    //reset the board, then make the moves on the stack to create the correct position
-    m_currentNode = moveId;
-    m_currentBoard = m_startingBoard;
-    while(!moveStack.isEmpty())
-    {
-        m_currentBoard.doMove(moveStack.pop());
+        //reset the board, then make the moves on the stack to create the correct position
+        m_currentNode = moveId;
+        m_currentBoard = m_startingBoard;
+        while(!moveStack.isEmpty())
+        {
+            m_currentBoard.doMove(moveStack.pop());
+        }
     }
 
     indicateAnnotationsOnBoard(moveId);
@@ -1690,8 +1599,6 @@ void Game::removeNode(MoveId moveId)
     {
         m_variationStartAnnotations.remove(node);
         m_annotations.remove(node);
-        m_squareAnnotations.remove(node);
-        m_arrowAnnotations.remove(node);
 
         if(variationCount(node))
         {
@@ -1701,9 +1608,11 @@ void Game::removeNode(MoveId moveId)
             }
         }
         removeNode(m_moveNodes[node].nextNode);
-        if(!atLineStart(node))
+        MoveId prevNode = m_moveNodes[node].previousNode;
+
+        if(!m_moveNodes[prevNode].variations.contains(node))
         {
-            m_moveNodes[m_moveNodes[node].previousNode].nextNode = NO_MOVE;
+            m_moveNodes[prevNode].nextNode = NO_MOVE;
         }
         m_moveNodes[node].remove();
     }
@@ -1714,8 +1623,6 @@ void Game::clear()
     m_moveNodes.clear();
     m_variationStartAnnotations.clear();
     m_annotations.clear();
-    m_squareAnnotations.clear();
-    m_arrowAnnotations.clear();
 
     m_startPly = 0;
     m_currentNode = 0;
@@ -1751,11 +1658,11 @@ void Game::removeTag(const QString& tag)
     m_tags.remove(tag);
 }
 
-void Game::setStartingBoard(const Board& startingBoard)
+void Game::setStartingBoard(const Board& startingBoard, QString text)
 {
     Game state = *this;
     dbSetStartingBoard(startingBoard.toFen());
-    emit signalGameModified(true, state, tr("Set starting board"));
+    emit signalGameModified(true, state, text);
 }
 
 void Game::dbSetStartingBoard(const Board& startingBoard)
@@ -1881,8 +1788,6 @@ void Game::dumpAnnotations(MoveId moveId) const
     {
         qDebug() << "   Annotations";
         qDebug() << "     Text        : " << m_annotations.value(moveId);
-        qDebug() << "     Square      : " << m_squareAnnotations.value(moveId);
-        qDebug() << "     Arrow       : " << m_arrowAnnotations.value(moveId);
         qDebug() << "     Start       : " << m_variationStartAnnotations.value(moveId);
     }
 }
@@ -1900,6 +1805,7 @@ void Game::dumpAllMoveNodes() const
     int moves, comments, nags;
     moveCount(&moves, &comments, &nags);
     qDebug() << "Moves: " << moves << " Comments: " << comments << " Nags: " << nags << endl;
+    qDebug() << "----------------------------------" << endl;
 }
 
 MoveId Game::findPosition(const Board& position) const
@@ -1967,14 +1873,10 @@ void Game::compact()
     {
         m_variationStartAnnotations.remove(m);
         m_annotations.remove(m);
-        m_squareAnnotations.remove(m);
-        m_arrowAnnotations.remove(m);
     }
 
     AnnotationMap variationStartAnnotations;
     AnnotationMap annotations;
-    AnnotationMap squareAnnotations;
-    AnnotationMap arrowAnnotations;
 
     foreach(MoveId key, oldIdNewIdMapping.keys())
     {
@@ -1988,8 +1890,6 @@ void Game::compact()
 
         GAME_UPDATE_ANNOT(variationStartAnnotations, m_variationStartAnnotations);
         GAME_UPDATE_ANNOT(annotations, m_annotations);
-        GAME_UPDATE_ANNOT(squareAnnotations, m_squareAnnotations);
-        GAME_UPDATE_ANNOT(arrowAnnotations, m_arrowAnnotations);
     }
 
     // update nodes links to other nodes in shrinked list (prev, next, variations)
@@ -2006,18 +1906,15 @@ void Game::compact()
             GAME_UPDATE_MOVEID(vars[j]);
         }
         vars.removeAll(NO_MOVE);
+        vars.removeAll(0);
 #undef GAME_UPDATE_MOVEID
     }
 
     m_variationStartAnnotations.clear();
     m_annotations.clear();
-    m_squareAnnotations.clear();
-    m_arrowAnnotations.clear();
 
     m_variationStartAnnotations = variationStartAnnotations;
     m_annotations               = annotations;
-    m_squareAnnotations         = squareAnnotations;
-    m_arrowAnnotations          = arrowAnnotations;
 
     m_moveNodes.clear();
     m_moveNodes = moveNodes;
@@ -2073,23 +1970,33 @@ void Game::reparentVariation(MoveId variation, MoveId parent)
     }
 }
 
-QString Game::specialAnnotation(QString& annotation, QString specialMark) const  // [%csl  [%cal
+int Game::compareTags(const Game& game) const
 {
-    QString result = "";
-    int specialAnnotationStart = annotation.indexOf(specialMark);
-    // If we found a chessbase special annotation
-    if(specialAnnotationStart >= 0)
+    if (m_tags.count() < game.m_tags.count())
     {
-        int specialAnnotationDataStart = specialAnnotationStart + specialMark.length() + 1; //+ 1 for space after specialMark
-        int specialAnnotationEnd = annotation.indexOf(']', specialAnnotationDataStart);
-
-        if(specialAnnotationEnd > specialAnnotationDataStart)
-        {
-            result = annotation.mid(specialAnnotationDataStart, specialAnnotationEnd - specialAnnotationDataStart);
-            annotation.remove(QRegExp(QString("\\") + specialMark + "[\\]]*\\]"));
-        };
+        return -1;
     }
-    return result;
+    if (m_tags.count() > game.m_tags.count())
+    {
+        return +1;
+    }
+    return (m_tags == game.m_tags);
+}
+
+int Game::compareMoves(const Game& /*game*/) const
+{
+    // todo
+    return 0;
+}
+
+int Game::compare(const Game& game) const
+{
+    int retVal = compareTags(game);
+    if (retVal == 0)
+    {
+        retVal = compareMoves(game);
+    }
+    return retVal;
 }
 
 
