@@ -79,11 +79,24 @@ public :
         NextMove,
         PreviousMove
     };
+
+    static const QRegExp egt;
+    static const QRegExp clk;
+    static const QRegExp emt;
+    static const QRegExp csl;
+    static const QRegExp cal;
+
     Game();
     Game(const Game& game);
     Game& operator=(const Game& game);
     virtual ~Game();
     // **** Querying game information ****
+    /** compare tags */
+    int compareTags(const Game& game) const;
+    /** compare moves */
+    int compareMoves(const Game& game) const;
+    /** compare games */
+    int compare(const Game& game) const;
     /** @return current position */
     const Board& board() const;
     /** @return current position in FEN */
@@ -111,11 +124,7 @@ public :
     /** @return arrowAnnotation at move at node @p moveId. */
     QString arrowAnnotation(MoveId moveId = CURRENT_MOVE) const;
     /** @return annotation at move at node @p moveId. */
-    QString egtAnnotation(MoveId moveId = CURRENT_MOVE) const;
-    /** @return annotation at move at node @p moveId. */
-    QString emtAnnotation(MoveId moveId = CURRENT_MOVE) const;
-    /** @return annotation at move at node @p moveId. */
-    QString clkAnnotation(MoveId moveId = CURRENT_MOVE) const;
+    QString specAnnotation(const QRegExp &r, MoveId moveId = CURRENT_MOVE) const;
     /** @return time annotation (either egt or clock) at move at node @p moveId. */
     QString timeAnnotation(MoveId moveId = CURRENT_MOVE) const;
 
@@ -259,11 +268,11 @@ public :
     bool mergeAsVariation(Game &otherGame);
     /** Merge @p otherGame variations starting from otherGames current position into this game as new variations */
     bool mergeVariations(Game &otherGame);
-    /** Find the point in the this game where @p otherGame fits in the next time.
-        @retval Node from where the merging shall start in other game */
-    MoveId findMergePoint(const Game &otherGame);
+
     /** Merge Game @p g into this game */
     void mergeWithGame(const Game& g);
+    /** Merge Game @p g into this game */
+    void dbMergeWithGame(const Game& g);
     /** Promotes the given variation to the main line, returns true if successful */
     bool promoteVariation(MoveId variation);
     /** Removes the given variation, returns true if successful */
@@ -271,8 +280,6 @@ public :
     /** Removes all variations and mainline moves after the current position,
     * or before the current position if @p position == BeforeMove */
     void truncateVariation(Position position = AfterMove);
-    /** Find the next illegal position in all variations and mainline moves after the current position, and cut the game from there */
-    void truncateVariationAfterNextIllegalPosition();
     /** Removes all tags and moves */
     void clear();
     /** Set the game start position */
@@ -334,18 +341,27 @@ public :
     void moveVariationUp(MoveId moveId);
     /** Move the variation @p moveId of the curent node down in the list of variations */
     void moveVariationDown(MoveId moveId);
+    /** Copy in a game and set it as modified (different from operator=) */
+    void copyFromGame(const Game& g);
 
     int resultAsInt() const;
-    void setStartingBoard(const Board &startingBoard);
+    void setStartingBoard(const Board &startingBoard, QString text);
+
+protected:
+    /** Find the point in the this game where @p otherGame fits in the next time.
+        @retval Node from where the merging shall start in other game */
+    MoveId findMergePoint(const Game &otherGame);
+    /** Promotes the given variation to the main line */
+    void dbPromoteVariation(MoveId variation);
+    /** Find the next illegal position in all variations and mainline moves after the current position, and cut the game from there */
+    void truncateVariationAfterNextIllegalPosition();
+
 signals:
     void signalGameModified(bool,Game,QString);
     void signalMoveChanged();
 
 private:
 
-    QString specialAnnotation(QString& annotation, QString specialMark) const;  // [%csl  [%cal
-    QString composeAnnotation(QString freeText, MoveId node) const;
-
 #pragma pack(push,2)
     struct MoveNode
     {
@@ -390,12 +406,6 @@ private:
     /** Annotations for move nodes */
     AnnotationMap m_annotations;
 
-    /** SquareAnnotations for move nodes */
-    AnnotationMap m_squareAnnotations;
-
-    /** ArrowAnnotations for move nodes */
-    AnnotationMap m_arrowAnnotations;
-
     /** Map keeping pgn tags of the game */
     QMap<QString, QString> m_tags;
 
