@@ -80,6 +80,19 @@ void Index::setTag(const QString& tagName, const QString& value, GameId gameId)
     m_mapTagToIndexItems.insertMulti(tagIndex, gameId);
 }
 
+void Index::removeTag(const QString& tagName, GameId gameId)
+{
+    if(m_tagNameIndex.contains(tagName))
+    {
+        TagIndex tagIndex = m_tagNameIndex.value(tagName);
+        if((int)gameId < m_indexItems.count())
+        {
+            m_indexItems[gameId]->remove(tagIndex);
+            m_mapTagToIndexItems.remove(tagIndex, gameId);
+        }
+    }
+}
+
 void Index::setValidFlag(GameId gameId, bool value)
 {
     m_validFlags[gameId] = value;
@@ -264,7 +277,20 @@ QBitArray Index::listInRange(const QString& tagName, const QString& minValue, co
     for(int i = 0; i < count(); ++i)
     {
         QString value = tagValue(tagIndex, i);
-        list.setBit(i, (minValue < value) && (value < maxValue));
+        list.setBit(i, (minValue <= value) && (value <= maxValue));
+    }
+    return list;
+}
+
+QBitArray Index::listInRange(const QString &tagName, int minValue, int maxValue) const
+{
+    TagIndex tagIndex = m_tagNameIndex.value(tagName);
+
+    QBitArray list(count(), false);
+    for(int i = 0; i < count(); ++i)
+    {
+        int value = tagValue(tagIndex, i).toInt();
+        list.setBit(i, (minValue <= value) && (value <= maxValue));
     }
     return list;
 }
