@@ -17,7 +17,6 @@
 #include <QObject>
 
 #include "indexitem.h"
-#include "search.h"
 #include "game.h"
 
 /** @ingroup Database
@@ -80,6 +79,9 @@ public:
     /** Returns a bit array to indicate which games in index have a tag value in given range */
     QBitArray listInRange(const QString& tag, const QString& minValue, const QString& maxValue) const;
 
+    /** Returns a bit array to indicate which games in index have a tag value in given range */
+    QBitArray listInRange(const QString& tag, int minValue, int maxValue) const;
+
     /** Returns a bit array to indicate which games in index have a tag value which somewhat matches */
     QBitArray listPartialValue(const QString& tagName, const QString& value) const;
 
@@ -108,7 +110,8 @@ public:
     bool deleted(GameId gameId) const;
     /** Set delete flag */
     void setDeleted(GameId gameId, bool df);
-
+    /* Remove a tag from the index  do not care for the value */
+    void removeTag(const QString &tagName, GameId gameId);
 signals:
     void progress(int);
 
