@@ -35,17 +35,15 @@ PgnDatabase::~PgnDatabase()
 
 bool PgnDatabase::open(const QString& filename, bool utf8)
 {
-    if(m_isOpen)
+    if(!m_file)
     {
-        return false;
-    }
-    m_break = false;
-    m_filename = filename;
-    if(openFile(filename))
-    {
-        m_isOpen = true;
-        m_utf8 = utf8;
-        return true;
+        m_break = false;
+        m_filename = filename;
+        if(openFile(filename))
+        {
+            m_utf8 = utf8;
+            return true;
+        }
     }
     return false;
 }
@@ -109,7 +107,7 @@ bool PgnDatabase::readOffsetFile(const QString& filename, volatile bool *breakFl
         return false;
     }
 
-    int streamVersion = QDataStream::Qt_4_7;
+    int streamVersion = QDataStream::Qt_4_6;
     if(version > 0)
     {
         in >> streamVersion;
@@ -321,7 +319,6 @@ bool PgnDatabase::openString(const QString& content)
     //open file
     initialise();
     m_filename = "Internal.pgn";
-    m_isOpen = true;
     QByteArray byteArray;
     byteArray.append(content.toLatin1());
     QBuffer* buffer = new QBuffer(&byteArray);
@@ -331,6 +328,11 @@ bool PgnDatabase::openString(const QString& content)
     return parseFile();
 }
 
+quint64 PgnDatabase::count() const
+{
+    return m_count;
+}
+
 QString PgnDatabase::filename() const
 {
     return m_filename;
@@ -352,42 +354,42 @@ void PgnDatabase::close()
     initialise();
 }
 
-void PgnDatabase::loadGameMoves(int index, Game& game)
+void PgnDatabase::loadGameMoves(GameId gameId, Game& game)
 {
-    if(!m_isOpen || index >= m_count)
+    QMutexLocker m(&m_mutex);
+    if(!m_file || gameId >= m_count)
     {
         return;
     }
     game.clear();
-    seekGame(index);
+    seekGame(gameId);
     skipTags();
-    QString fen = m_index.tagValue(TagNameFEN, index); // was m_count -1
+    QString fen = m_index.tagValue(TagNameFEN, gameId); // was m_count -1
     if(fen != "?")
     {
-        game.setStartingBoard(fen);
+        game.dbSetStartingBoard(fen);
     }
     parseMoves(&game);
 }
 
-bool PgnDatabase::loadGame(int index, Game& game)
+bool PgnDatabase::loadGame(GameId gameId, Game& game)
 {
-    if(!m_isOpen || index >= m_count)
+    if(!m_file || gameId >= m_count)
     {
         return false;
     }
-    lock();
+    QMutexLocker m(&m_mutex);
     //parse the game
     game.clear();
-    loadGameHeaders(index, game);
-    seekGame(index);
+    loadGameHeaders(gameId, game);
+    seekGame(gameId);
     skipTags();
-    QString fen = m_index.tagValue(TagNameFEN, index);  // was m_count - 1
+    QString fen = m_index.tagValue(TagNameFEN, gameId);  // was m_count - 1
     if(fen != "?")
     {
-        game.setStartingBoard(fen);
+        game.dbSetStartingBoard(fen);
     }
     parseMoves(&game);
-    unlock();
 
     return m_variation != -1 || fen != "?";  // Not sure of all of the ramifications of this
     // but it seeems to fix the problem with FENs
@@ -399,14 +401,12 @@ void PgnDatabase::initialise()
     m_gameOffsets64 = 0;
     m_gameOffsets32 = 0;
     m_inComment = false;
-    m_isOpen = false;
+    m_inPreComment = false;
     m_filename = QString();
     m_count = 0;
     m_allocated = 0;
 }
 
-
-
 void PgnDatabase::addOffset()
 {
     IndexBaseType fp = m_file->pos();
@@ -453,9 +453,9 @@ inline void PgnDatabase::skipLine()
     m_lineBuffer = m_file->readLine();
 }
 
-void PgnDatabase::seekGame(int index)
+void PgnDatabase::seekGame(GameId gameId)
 {
-    IndexBaseType n = offset(index);
+    IndexBaseType n = offset(gameId);
     if(!m_file->seek(n))
     {
         qDebug() << "Seeking offset " << n << " failed!";
@@ -521,6 +521,7 @@ bool PgnDatabase::parseMoves(Game* game)
 {
     m_gameOver = false;
     m_inComment = false;
+    m_inPreComment = false;
     m_comment.clear();
     m_precomment.clear();
     m_newVariation = false;
@@ -549,8 +550,9 @@ bool PgnDatabase::parseMoves(Game* game)
         {
             if(!m_precomment.isEmpty())
             {
-                game->setAnnotation(m_precomment);
+                game->dbSetAnnotation(m_precomment);
                 m_precomment.clear();
+                m_inPreComment = false;
             }
         }
     }
@@ -609,14 +611,15 @@ inline void PgnDatabase::parseDefaultToken(Game* game, QString token)
         {
             nag = VeryGoodMove;
         }
-        else if(token.endsWith("!?"))
+        else if(token.endsWith("?!"))
         {
-            nag = SpeculativeMove;
+            nag = QuestionableMove;
         }
         else
         {
             nag = GoodMove;
         }
+        token = token.section('!',	0, 0);
     }
     else if(token.endsWith("?"))
     {
@@ -624,14 +627,15 @@ inline void PgnDatabase::parseDefaultToken(Game* game, QString token)
         {
             nag = VeryPoorMove;
         }
-        else if(token.endsWith("?!"))
+        else if(token.endsWith("!?"))
         {
-            nag = QuestionableMove;
+            nag = SpeculativeMove;
         }
         else
         {
             nag = PoorMove;
         }
+        token = token.section('?',	0, 0);
     }
 
     if(!token.isEmpty())
@@ -639,21 +643,23 @@ inline void PgnDatabase::parseDefaultToken(Game* game, QString token)
         if(m_newVariation)
         {
             game->backward();
-            m_variation = game->addVariation(token, QString(), nag);
+            m_variation = game->dbAddVariation(token, QString(), nag);
             if(!m_precomment.isEmpty())
             {
-                game->setAnnotation(m_precomment, m_variation, Game::BeforeMove);
+                game->dbSetAnnotation(m_precomment, m_variation, Game::BeforeMove);
                 m_precomment.clear();
+                m_inPreComment = false;
             }
             m_newVariation = false;
         }
         else  	// First move in the game
         {
-            m_variation = game->addMove(token, QString(), nag);
+            m_variation = game->dbAddMove(token, QString(), nag);
             if(!m_precomment.isEmpty())
             {
-                game->setAnnotation(m_precomment, m_variation, Game::BeforeMove);
+                game->dbSetAnnotation(m_precomment, m_variation, Game::BeforeMove);
                 m_precomment.clear();
+                m_inPreComment = false;
             }
         }
     }
@@ -661,6 +667,7 @@ inline void PgnDatabase::parseDefaultToken(Game* game, QString token)
 
 void PgnDatabase::parseToken(Game* game, const QString& token)
 {
+    // qDebug() << "Parsing Token:" << token << ":";
     switch(token.at(0).toLatin1())
     {
     case '(':
@@ -674,7 +681,6 @@ void PgnDatabase::parseToken(Game* game, const QString& token)
         break;
     case '{':
         m_comment.clear();
-        m_precomment.clear();
         m_inComment = true;
         m_currentLine = m_currentLine.right((m_currentLine.length() - m_pos) - 1);
         break;
@@ -785,11 +791,17 @@ void PgnDatabase::parseComment(Game* game)
         m_inComment = false;
         if(m_newVariation || game->plyCount() == 0)
         {
+            if (game->plyCount()==0 && m_inPreComment)
+            {
+                game->dbSetAnnotation(m_precomment, 0);
+                m_inPreComment = false;
+            }
             m_precomment = m_comment.trimmed();
+            m_inPreComment = true;
         }
         else
         {
-            game->setAnnotation(m_comment.trimmed());
+            game->dbSetAnnotation(m_comment.trimmed());
         }
         m_currentLine = m_currentLine.right((m_currentLine.length() - end) - 1);
     }
