@@ -16,8 +16,7 @@
 #include "guessengine.h"
 #include "recog.h"
 
-namespace Guess
-{
+using namespace Guess;
 
 // The Engine class implements the Scid built-in chess engine.
 // See guessengine.h for details.
@@ -55,7 +54,7 @@ static const int KingTrapsRook     =  35;  // E.g. King on f1, Rook on h1
 static const int DoubledPawn       =   8;
 static const int IsolatedPawn      =  16;
 static const int BackwardPawn      =  10;  // Pawn at base of pawn chain.
-static const int DispersedPawn     =  10;  // Not in pawn chain/duo. (Unused)
+//static const int DispersedPawn     =  10;  // Not in pawn chain/duo. (Unused)
 static const int BlockedHomePawn   =  15;  // Blocked pawn on d2/e2/d7/e7.
 static const int BishopPair        =  25;  // Pair of bishops.
 static const int BishopEyesKing    =  12;  // Bishop targets enemy king.
@@ -1800,21 +1799,21 @@ Engine::Think(MoveList * mlist)
     ClearHistoryValues();
 
     // If no legal move list was specified, generate and search all moves:
-    if(mlist == NULL || !mlist->Size())
+    if(mlist == NULL || !mlist->size())
     {
         Pos.GenerateMoves();
         mlist = Pos.GetLegalMoves();
     }
 
     // No legal moves? Return 0 for stalemate, -Infinity for checkmate.
-    if(mlist->Size() == 0)
+    if(mlist->size() == 0)
     {
         return (Pos.IsKingInCheck() ? -Infinity : 0);
     }
 
     // Sort the root move list by quiescent evaluation to get a
     // reasonably good initial move order:
-    for(unsigned int i = 0; i < mlist->Size(); i++)
+    for(unsigned int i = 0; (int) i < mlist->size(); i++)
     {
         simpleMoveT * sm = mlist->Get(i);
         DoMove(sm);
@@ -1825,7 +1824,7 @@ Engine::Think(MoveList * mlist)
 
     // Check for an easy move, one that scores more than two pawns
     // better than any alternative:
-    if(mlist->Size() > 1)
+    if(mlist->size() > 1)
     {
         int margin = mlist->Get(0)->score - mlist->Get(1)->score;
         if(margin > (2 * PawnValue))
@@ -1954,15 +1953,15 @@ Engine::SearchRoot(int depth, int alpha, int beta, MoveList * mlist)
 
     // No legal moves to search? Just return an equal score for
     // stalemate or -Infinity for checkmate.
-    if(mlist->Size() == 0)
+    if(mlist->size() == 0)
     {
         return (Pos.IsKingInCheck() ? -Infinity : 0);
     }
 
-    bool isOnlyMove = (mlist->Size() == 1);
+    bool isOnlyMove = (mlist->size() == 1);
     int bestScore = -Infinity - 1;
 
-    for(unsigned int movenum = 0; movenum < mlist->Size(); movenum++)
+    for(unsigned int movenum = 0; (int) movenum < mlist->size(); movenum++)
     {
         simpleMoveT * sm = mlist->Get(movenum);
         unsigned int oldNodeCount = NodeCount;
@@ -2213,7 +2212,7 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
     {
         gotHashMove = true;
         // For now, we only add the hash move to the move list.
-        mlist.Add(&hashmove);
+        mlist.append(hashmove);
         mlist.Get(0)->score = ENGINE_HASH_SCORE;
     }
     else
@@ -2222,7 +2221,7 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
         gotHashMove = false;
         Pos.GenerateMoves(&mlist, EMPTY, GEN_ALL_MOVES, InCheck[Ply]);
         ScoreMoves(&mlist);
-        isOnlyMove = (mlist.Size() == 1);
+        isOnlyMove = (mlist.size() == 1);
     }
 
     // If there is only one legal move, extend the search:
@@ -2236,7 +2235,7 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
     int bestMoveIndex = -1;
 
     // Search each move:
-    for(unsigned int movenum = 0; movenum < mlist.Size(); movenum++)
+    for(unsigned int movenum = 0; (int) movenum < mlist.size(); movenum++)
     {
         // Find the highest-scoring remaining move:
         mlist.FindBest(movenum);
@@ -2365,7 +2364,7 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
         // the start of the list so it does not get searched again.
         if(movenum == 0  &&  gotHashMove  &&  !isOnlyMove)
         {
-            mlist.Clear();
+            mlist.clear();
             Pos.GenerateMoves(&mlist, EMPTY, GEN_ALL_MOVES, InCheck[Ply]);
             ScoreMoves(&mlist);
             int hashIndex = mlist.Find(&hashmove);
@@ -2382,7 +2381,7 @@ Engine::Search(int depth, int alpha, int beta, bool tryNullMove)
         }
     }
 
-    if(mlist.Size() == 0)
+    if(mlist.size() == 0)
     {
         // No legal moves? Must be checkmate or stalemate:
         return (InCheck[Ply] ? (-Infinity + Ply) : 0);
@@ -2493,7 +2492,7 @@ Engine::Quiesce(int alpha, int beta)
     // Generate and score the list of captures:
     MoveList mlist;
     Pos.GenerateMoves(&mlist, GEN_CAPTURES);
-    for(unsigned int m = 0; m < mlist.Size(); m++)
+    for(unsigned int m = 0; (int) m < mlist.size(); m++)
     {
         simpleMoveT * sm = mlist.Get(m);
         sm->score = SEE(sm->from, sm->to);
@@ -2502,7 +2501,7 @@ Engine::Quiesce(int alpha, int beta)
     // Iterate through each quiescent move to find a beta cutoff or
     // improve the alpha score:
 
-    for(unsigned int i = 0; i < mlist.Size(); i++)
+    for(unsigned int i = 0; (int) i < mlist.size(); i++)
     {
         // Find the highest-scoring remaining move, make it and search:
         mlist.FindBest(i);
@@ -2949,7 +2948,7 @@ Engine::SEE(squareT from, squareT target)
 void
 Engine::ScoreMoves(MoveList * mlist)
 {
-    for(unsigned int i = 0; i < mlist->Size(); i++)
+    for(unsigned int i = 0; (int) i < mlist->size(); i++)
     {
         simpleMoveT * sm = mlist->Get(i);
         if(sm->capturedPiece != EMPTY  ||  sm->promote != EMPTY)
@@ -3116,7 +3115,7 @@ Engine::PerfTest(unsigned int depth)
     MoveList mlist;
     Pos.GenerateMoves(&mlist);
     unsigned int nmoves = 0;
-    for(unsigned int i = 0; i < mlist.Size(); i++)
+    for(unsigned int i = 0; (int) i < mlist.size(); i++)
     {
         simpleMoveT * sm = mlist.Get(i);
         Pos.DoSimpleMove(sm);
@@ -3126,9 +3125,22 @@ Engine::PerfTest(unsigned int depth)
     return nmoves;
 }
 
-} // End namespace Guess
+void Engine::UpdatePV(simpleMoveT * sm)
+{
+    if(Ply >= ENGINE_MAX_PLY - 1)
+    {
+        return;
+    }
+    if(InNullMove > 0)
+    {
+        return;
+    }
 
-//////////////////////////////////////////////////////////////////////
-//  EOF: engine.cpp
-//////////////////////////////////////////////////////////////////////
+    PV[Ply].move[Ply] = *sm;
+    for(unsigned int j = Ply + 1; j < PV[Ply + 1].length; j++)
+    {
+        PV[Ply].move[j] = PV[Ply + 1].move[j];
+    }
+    PV[Ply].length = PV[Ply + 1].length;
+}
 
