@@ -16,7 +16,6 @@
 #include <QLCDNumber>
 #include <QToolBar>
 
-
 ChessBrowser::ChessBrowser(QWidget *p, bool showGameMenu) : QTextBrowser(p), toolBar(0), m_gameMenu(NULL)
 {
     setObjectName("ChessBrowser");
@@ -301,7 +300,10 @@ void ChessBrowser::dropEvent(QDropEvent *event)
     const GameMimeData* gameMimeData = qobject_cast<const GameMimeData*>(mimeData);
     if(gameMimeData)
     {
-        mergeGame(gameMimeData->m_index);
+        foreach(int index, gameMimeData->m_indexList)
+        {
+            mergeGame(index);
+        }
     }
 
     event->acceptProposedAction();
