@@ -46,6 +46,9 @@ void CopyDialog::setMode(SrcMode mode)
     case SingleGame:
         ui.singleButton->setChecked(true);
         break;
+    case Selection:
+        ui.selectedButton->setChecked(true);
+        break;
     case Filter:
         ui.filterButton->setChecked(true);
         break;
@@ -66,6 +69,10 @@ int CopyDialog::getMode() const
     {
         return SingleGame;
     }
+    else if(ui.selectedButton->isChecked())
+    {
+        return Selection;
+    }
     else if(ui.filterButton->isChecked())
     {
         return Filter;
