@@ -6,7 +6,6 @@
 #define DATABASELIST_H
 
 #include "tableview.h"
-#include "game.h"
 
 class QSortFilterProxyModel;
 class DatabaseListModel;
@@ -51,7 +50,7 @@ protected:
     void dragLeaveEvent(QDragLeaveEvent *event);
     void dropEvent(QDropEvent *event);
 
-    void appendGameToDataBase(QModelIndex index, int gameIndex);
+    void appendGameToDataBase(QModelIndex index, QList<int> gameIndexList);
     void appendDataBaseToDataBase(QPoint pos, QString src);
 
 private slots:
