@@ -22,14 +22,18 @@ class EngineOptionDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit EngineOptionDialog(QWidget *parent, EngineList& engineList, int index, Qt::WindowFlags f = 0);
+    explicit EngineOptionDialog(QWidget *parent, EngineList& engineList, int index);
     ~EngineOptionDialog();
     OptionValueList GetResults() const;
+
 public slots:
     void accept();
+    void reject();
+
 protected slots:
     void engineActivated();
     void optionReset();
+    void restoreLayout();
 
 private:
     Ui::EngineOptionDialog *ui;
