@@ -30,13 +30,16 @@
 
 GameList::GameList(Filter* filter, QWidget* parent) : TableView(parent)
 {
+    setSelectionMode(QAbstractItemView::ExtendedSelection);
     setObjectName("GameList");
     setWindowTitle(tr("Game list"));
-    //QSortFilterProxyModel* sortModel = new QSortFilterProxyModel(this);
     m_model = new FilterModel(filter, this);
-    //sortModel->setSourceModel(m_model);
-    //setModel(sortModel);
-    setModel(m_model);
+
+    sortModel = new GameListSortModel(this);
+    sortModel->setSourceModel(m_model);
+    sortModel->setDynamicSortFilter(false);
+    setModel(sortModel);
+
     connect(this, SIGNAL(clicked(const QModelIndex&)), SLOT(itemSelected(const QModelIndex&)));
     connect(this, SIGNAL(activated(const QModelIndex&)), SLOT(itemSelected(const QModelIndex&)));
     connect(this, SIGNAL(customContextMenuRequested(const QPoint&)), SLOT(slotContextMenu(const QPoint&)));
@@ -47,41 +50,153 @@ GameList::GameList(Filter* filter, QWidget* parent) : TableView(parent)
 #else
     horizontalHeader()->setSectionsClickable(true);
 #endif
-    connect(horizontalHeader(), SIGNAL(sectionClicked(int)), SLOT(simpleSearch(int)));
 
     slotReconfigure();
 
-    setSortingEnabled(false);
+    horizontalHeader()->setSortIndicatorShown(true);
+    setSortingEnabled(true);
+
     setDragEnabled(true);
 }
 
 
 GameList::~GameList()
 {
+    delete sortModel;
     delete m_model;
 }
 
+void GameList::ShowContextMenu(const QPoint& pos)
+{
+    QMenu headerMenu;
+    QAction* filterTag = headerMenu.addAction(tr("Find tag..."));
+    headerMenu.addSeparator();
+    QAction* hide = headerMenu.addAction(tr("Hide Column"));
+    headerMenu.addSeparator();
+    QAction* resizeAll = headerMenu.addAction(tr("Resize visible Columns"));
+    QAction* showAll = headerMenu.addAction(tr("Show all Columns"));
+
+    QAction* selectedItem = headerMenu.exec(mapToGlobal(pos));
+    int column = columnAt(pos.x());
+    if (selectedItem == filterTag)
+    {
+        simpleSearch(column);
+    }
+    else if(selectedItem == hide)
+    {
+        if(column > 0)
+        {
+            hideColumn(column);
+        }
+    }
+    else if(selectedItem == showAll)
+    {
+        for(int i = 0; i < model()->columnCount(); ++i)
+        {
+            showColumn(i);
+        }
+    }
+    else if (selectedItem == resizeAll)
+    {
+        resizeColumnsToContents();
+    }
+}
+
+void GameList::removeSelection()
+{
+    clearSelection();
+    emit signalFirstGameLoaded(m_model->filter()->count()>0);
+    emit signalLastGameLoaded(m_model->filter()->count()>0);
+}
+
 void GameList::slotItemSelected(const QModelIndex& index)
 {
-    m_index = index;
     scrollTo(index, EnsureVisible);
 }
 
+QModelIndex GameList::NewSortIndex(int row) const
+{
+    if (sortModel)
+    {
+        QModelIndex m = sortModel->index(row,0);
+        return m;
+    }
+    return m_model->index(row,0);
+}
+
+QModelIndex GameList::GetSourceIndex(const QModelIndex& index) const
+{
+    if (sortModel)
+    {
+        QModelIndex m = sortModel->mapToSource(index);
+        return m;
+    }
+    return index;
+}
+
+QModelIndex GameList::GetSortModelIndex(const QModelIndex& index) const
+{
+    if (sortModel)
+    {
+        QModelIndex m = sortModel->mapFromSource(index);
+        return m;
+    }
+    return index;
+}
+
 void GameList::itemSelected(const QModelIndex& index)
 {
-    emit selected(m_model->filter()->indexToGame(index.row()));
+    if (selectionModel()->selectedRows().size() == 1)
+    {
+        QModelIndex m = GetSourceIndex(index);
+        emit selected(m_model->filter()->indexToGame(m.row()));
+    }
+}
+
+void GameList::selectPreviousGame()
+{
+    QModelIndex sortIndex = currentIndex();
+    int row = std::max(0, sortIndex.row()-1);
+    QModelIndex sourceIndex = GetSourceIndex(NewSortIndex(row));
+    emit selected(m_model->filter()->indexToGame(sourceIndex.row()));
+}
+
+void GameList::selectNextGame()
+{
+    QModelIndex sortIndex = currentIndex();
+    int row = std::min(m_model->filter()->count(), sortIndex.row()+1);
+    QModelIndex sourceIndex = GetSourceIndex(NewSortIndex(row));
+    emit selected(m_model->filter()->indexToGame(sourceIndex.row()));
 }
 
 void GameList::setFilter(Filter* filter)
 {
+    setSortingEnabled(false);
+    if (filter->count() > 4096)
+    {
+        setModel(m_model);
+        delete sortModel;
+        sortModel = 0;
+    }
+    else
+    {
+        if (!sortModel)
+        {
+            sortModel = new GameListSortModel(this);
+            sortModel->setSourceModel(m_model);
+            sortModel->setDynamicSortFilter(false);
+        }
+        setModel(sortModel);
+    }
+
     m_model->setFilter(filter);
+    setSortingEnabled(!!sortModel);
     emit raiseRequest();
 }
 
 void GameList::slotContextMenu(const QPoint& pos)
 {
     QModelIndex cell = indexAt(pos);
-    m_index = cell;
     QModelIndexList selection = selectedIndexes();
     // Make sure the right click occured on a cell!
     if(cell.isValid() && selection.contains(cell))
@@ -91,12 +206,14 @@ void GameList::slotContextMenu(const QPoint& pos)
         QMenu* mergeMenu = menu.addMenu(tr("Merge into current game"));
         mergeMenu->addAction(tr("All Games"), this, SLOT(slotMergeAllGames()));
         mergeMenu->addAction(tr("Filter"), this, SLOT(slotMergeFilter()));
-        mergeMenu->addAction(tr("Selected game"), this, SLOT(slotMergeGame()));
+        mergeMenu->addAction(tr("Selected games"), this, SLOT(slotMergeGame()));
         menu.addSeparator();
         QAction* deleteAction = menu.addAction(tr("Delete game"), this, SLOT(slotDeleteGame()));
         deleteAction->setCheckable(true);
         deleteAction->setEnabled(!m_model->filter()->database()->isReadOnly());
-        int n = m_model->filter()->indexToGame(cell.row());
+
+        QModelIndex index = GetSourceIndex(cell);
+        int n = m_model->filter()->indexToGame(index.row());
         deleteAction->setChecked(m_model->filter()->database()->deleted(n));
         menu.exec(mapToGlobal(pos));
     }
@@ -104,12 +221,12 @@ void GameList::slotContextMenu(const QPoint& pos)
 
 void GameList::simpleSearch(int tagid)
 {
-    //if (QApplication::queryKeyboardModifiers() & Qt::ShiftModifier)
-    //{
-    //    sortByColumn(tagid);
-    //    return;
-    //}
     QuickSearchDialog dlg(this);
+    for (int section = 0; section < m_model->columnCount(); ++section)
+    {
+        QString tag = m_model->headerData(section, Qt::Horizontal).toString();
+        dlg.addTag(tag);
+    }
 
     dlg.setTag(tagid);
     if(m_model->filter()->count() <= 1)
@@ -127,7 +244,7 @@ void GameList::simpleSearch(int tagid)
     {
         m_model->filter()->setAll(1);
     }
-    else if((dlg.tag() == 0) || (dlg.tag() == 7) || (dlg.tag() == 11))
+    else if(dlg.tag() == 0)
     {
         // filter by number
         Search* ns = new NumberSearch(m_model->filter()->database(), value);
@@ -146,7 +263,9 @@ void GameList::simpleSearch(int tagid)
         if ((list.size() > 1) && (dlg.tag() != 9))
         {
             // Filter a range
-            Search* ts = new TagSearch(m_model->filter()->database(), tag, list.at(0), list.at(1));
+            Search* ts = (dlg.tag() == 11) ?
+                    new TagSearch(m_model->filter()->database(), tag, list.at(0).toInt(), list.at(1).toInt()) :
+                    new TagSearch(m_model->filter()->database(), tag, list.at(0), list.at(1));
             if(dlg.mode())
             {
                 m_model->filter()->executeSearch(ts, Search::Operator(dlg.mode()));
@@ -220,8 +339,12 @@ void GameList::selectGame(int index)
     int i = m_model->filter()->gameToIndex(index);
     if(i != -1)
     {
-        setCurrentIndex(m_model->index(i, 0));
-        selectRow(i);
+        QModelIndex filterModelIndex = m_model->index(i, 0);
+        QModelIndex sortModelIndex = GetSortModelIndex(filterModelIndex);
+        setCurrentIndex(sortModelIndex);
+        selectRow(sortModelIndex.row());
+        emit signalFirstGameLoaded(sortModelIndex.row()==0);
+        emit signalLastGameLoaded(sortModelIndex.row()+1 == m_model->filter()->count());
     }
 }
 
@@ -233,7 +356,14 @@ void GameList::updateFilter()
 
 void GameList::slotCopyGame()
 {
-    emit requestCopyGame(m_model->filter()->indexToGame(m_index.row()));
+    QList<int> gameIndexList;
+    foreach(QModelIndex index, selectionModel()->selectedRows())
+    {
+        QModelIndex m = GetSourceIndex(index);
+        gameIndexList.append(m_model->filter()->indexToGame(m.row()));
+    }
+
+    emit requestCopyGame(gameIndexList);
 }
 
 void GameList::slotMergeAllGames()
@@ -248,29 +378,41 @@ void GameList::slotMergeFilter()
 
 void GameList::slotMergeGame()
 {
-    emit requestMergeGame(m_model->filter()->indexToGame(m_index.row()));
+    QList<int> gameIndexList;
+    foreach(QModelIndex index, selectionModel()->selectedRows())
+    {
+        QModelIndex m = GetSourceIndex(index);
+        gameIndexList.append(m_model->filter()->indexToGame(m.row()));
+    }
+
+    emit requestMergeGame(gameIndexList);
 }
 
 void GameList::slotDeleteGame()
 {
-    emit requestDeleteGame(m_model->filter()->indexToGame(m_index.row()));
+    QList<int> gameIndexList;
+    foreach(QModelIndex index, selectionModel()->selectedRows())
+    {
+        QModelIndex m = GetSourceIndex(index);
+        gameIndexList.append(m_model->filter()->indexToGame(m.row()));
+    }
+
+    emit requestDeleteGame(gameIndexList);
 }
 
-void GameList::startToDrag(const QModelIndex& index)
+void GameList::startDrag(Qt::DropActions /*supportedActions*/)
 {
     GameMimeData *mimeData = new GameMimeData;
-    mimeData->m_index = m_model->filter()->indexToGame(index.row());
+    foreach(QModelIndex index, selectionModel()->selectedRows())
+    {
+        QModelIndex m = GetSourceIndex(index);
+        mimeData->m_indexList.append(m_model->filter()->indexToGame(m.row()));
+    }
     QPixmap pixmap = style()->standardPixmap(QStyle::SP_FileIcon);
 
     QDrag* pDrag = new QDrag(this);
     pDrag->setMimeData(mimeData);
     pDrag->setPixmap(pixmap);
-    // pDrag->setHotSpot(hotSpot);
 
     pDrag->exec(Qt::CopyAction | Qt::MoveAction, Qt::CopyAction);
 }
-
-void GameList::startDrag(Qt::DropActions /*supportedActions*/)
-{
-    startToDrag(m_index);
-}
