@@ -20,6 +20,8 @@
 #include "tableview.h"
 #include "game.h"
 
+#include <QSortFilterProxyModel>
+
 class Filter;
 class FilterModel;
 
@@ -27,6 +29,12 @@ class FilterModel;
 The GameList class displays list of the games in current filter. It allows
 user to click on list header and perform simple tag searches. */
 
+class GameListSortModel : public QSortFilterProxyModel
+{
+public:
+    explicit GameListSortModel(QObject *parent = 0) : QSortFilterProxyModel(parent) {}
+};
+
 class GameList : public TableView
 {
     Q_OBJECT
@@ -34,6 +42,7 @@ public:
     GameList(Filter* filter, QWidget* parent = 0);
     ~GameList();
 
+    void removeSelection();
     /** Set current database */
 public slots:
     /** Change current filter/database */
@@ -54,7 +63,12 @@ public slots:
     void slotFilterListByEventPlayer(QString player, QString event);
     /** Select and show current player and ECO in the list */
     void slotFilterListByEcoPlayer(QString tag, QString eco, QString player);
-
+    /** Select the next game from the list */
+    void selectNextGame();
+    /** Select the previous game from the list */
+    void selectPreviousGame();
+    /** Show the context menu */
+    virtual void ShowContextMenu(const QPoint& pos);
 private slots:
     /** Re-emit the request to the receivers to perform some action */
     void itemSelected(const QModelIndex& index);
@@ -76,20 +90,24 @@ private slots:
 signals:
     void selected(int);
     void raiseRequest();
-    void requestMergeGame(int);
+    void requestMergeGame(QList<int>);
     void requestMergeAllGames();
     void requestMergeFilter();
-    void requestCopyGame(int);
-    void requestDeleteGame(int);
+    void requestCopyGame(QList<int>);
+    void requestDeleteGame(QList<int>);
     void requestGameData(Game&);
+    void signalFirstGameLoaded(bool);
+    void signalLastGameLoaded(bool);
 
 protected: //Drag'n'Drop Support
     void startDrag(Qt::DropActions supportedActions);
-    void startToDrag(const QModelIndex&);
+    QModelIndex GetSourceIndex(const QModelIndex& index) const;
+    QModelIndex NewSortIndex(int row) const;
+    QModelIndex GetSortModelIndex(const QModelIndex& index) const;
 
 private:
     FilterModel* m_model;
-    QModelIndex m_index;
+    GameListSortModel* sortModel;
 };
 
 #endif
