@@ -21,9 +21,10 @@
 #include "ecothread.h"
 #include "eventlistwidget.h"
 #include "exclusiveactiongroup.h"
-#include "filtermodel.h"
+#include "ficsclient.h"
 #include "game.h"
 #include "gamelist.h"
+#include "GameMimeData.h"
 #include "helpbrowser.h"
 #include "kbaction.h"
 #include "loadquery.h"
@@ -34,13 +35,14 @@
 #include "output.h"
 #include "pgndatabase.h"
 #include "playerlistwidget.h"
-#include "preferences.h"
 #include "quazip.h"
 #include "quazipfile.h"
 #include "savedialog.h"
 #include "settings.h"
+#include "style.h"
 #include "tablebase.h"
 #include "tableview.h"
+#include "tagdialog.h"
 #include "toolmainwindow.h"
 #include "translatingslider.h"
 #include "version.h"
@@ -64,6 +66,8 @@
 #include <QTimer>
 #include <QToolBar>
 
+#undef FICS_SUPPORT
+
 MainWindow::MainWindow() : QMainWindow(),
     m_tabDragIndex(-1),
     m_pDragTabBar(0),
@@ -73,10 +77,15 @@ MainWindow::MainWindow() : QMainWindow(),
     m_currentFrom(InvalidSquare),
     m_currentTo(InvalidSquare),
     m_machineHasToMove(false),
-    m_bInDrag(false)
+    m_bInDrag(false),
+    m_gameMode(false)
 {
     setObjectName("MainWindow");
 
+#ifdef FICS_SUPPORT
+    m_ficsClient = new FicsClient(this);
+#endif
+
     m_autoPlayTimer = new QTimer(this);
     m_autoPlayTimer->setInterval(AppSettings->getValue("/Board/AutoPlayerInterval").toInt());
     m_autoPlayTimer->setSingleShot(true);
@@ -96,6 +105,11 @@ MainWindow::MainWindow() : QMainWindow(),
     m_databases.append(pClipDB);
     m_currentDatabase = 0;
 
+    /* Game List */
+    DockWidgetEx* gameListDock = new DockWidgetEx(tr("Game List"), this);
+    gameListDock->setObjectName("GameList");
+    m_gameList = new GameList(databaseInfo()->filter(), gameListDock);
+
     /* Actions */
     setupActions();
 
@@ -169,16 +183,13 @@ MainWindow::MainWindow() : QMainWindow(),
     gameTextDock->setTitleBarWidget(m_gameTitle);
 
     /* Game List */
-    DockWidgetEx* gameListDock = new DockWidgetEx(tr("Game List"), this);
-    gameListDock->setObjectName("GameList");
-    m_gameList = new GameList(databaseInfo()->filter(), gameListDock);
     m_gameList->setMinimumSize(150, 100);
     connect(m_gameList, SIGNAL(selected(int)), SLOT(slotFilterLoad(int)));
-    connect(m_gameList, SIGNAL(requestCopyGame(int)), SLOT(slotDatabaseCopySingle(int)));
-    connect(m_gameList, SIGNAL(requestMergeGame(int)), SLOT(slotMergeActiveGame(int)));
+    connect(m_gameList, SIGNAL(requestCopyGame(QList<int>)), SLOT(slotDatabaseCopySingle(QList<int>)));
+    connect(m_gameList, SIGNAL(requestMergeGame(QList<int>)), SLOT(slotMergeActiveGame(QList<int>)));
     connect(m_gameList, SIGNAL(requestMergeAllGames()), SLOT(slotMergeAllGames()));
     connect(m_gameList, SIGNAL(requestMergeFilter()), SLOT(slotMergeFilter()));
-    connect(m_gameList, SIGNAL(requestDeleteGame(int)), SLOT(slotDatabaseDeleteGame(int)));
+    connect(m_gameList, SIGNAL(requestDeleteGame(QList<int>)), SLOT(slotDatabaseDeleteGame(QList<int>)));
     connect(m_gameList, SIGNAL(requestGameData(Game&)), SLOT(slotGetGameData(Game&)));
     connect(this, SIGNAL(reconfigure()), m_gameList, SLOT(slotReconfigure()));
     gameListDock->setWidget(m_gameList);
@@ -273,7 +284,7 @@ MainWindow::MainWindow() : QMainWindow(),
 
     /* Opening Tree */
     DockWidgetEx* openingDock = new DockWidgetEx(tr("Opening Tree"), this);
-    openingDock->setObjectName("OpeningTreeDock");
+    openingDock->setObjectName("    Dock");
     m_openingTreeWidget = new OpeningTreeWidget(this);
     openingDock->setWidget(m_openingTreeWidget);
     addDockWidget(Qt::RightDockWidgetArea, openingDock);
@@ -359,11 +370,11 @@ MainWindow::MainWindow() : QMainWindow(),
     m_sliderSpeed->setMultiplier(1000);
     m_sliderSpeed->setOrientation(Qt::Horizontal);
     m_sliderSpeed->setMinimum(0);
-    m_sliderSpeed->setMaximum(10);
+    m_sliderSpeed->setMaximum(60);
     m_sliderSpeed->setTranslatedValue(AppSettings->getValue("/Board/AutoPlayerInterval").toInt());
     m_sliderSpeed->setTickInterval(1);
-    m_sliderSpeed->setTickPosition(QSlider::TicksBothSides);
-    m_sliderSpeed->setMaximumWidth(300);
+    m_sliderSpeed->setTickPosition(QSlider::NoTicks);
+    m_sliderSpeed->setMaximumWidth(400);
     connect(m_sliderSpeed, SIGNAL(translatedValueChanged(int)), SLOT(slotMoveIntervalChanged(int)));
     connect(m_sliderSpeed, SIGNAL(translatedValueChanged(int)), m_mainAnalysis, SLOT(setMoveTime(int)));
     connect(m_sliderSpeed, SIGNAL(translatedValueChanged(int)), analysis, SLOT(setMoveTime(int)));
@@ -543,20 +554,23 @@ void MainWindow::evaluateSanNag(QKeyEvent *e)
     {
         // Try to figure out a SAN
         int matches = NagSet::prefixCount(m_nagText);
-        if(matches == 1)
+        if(matches >= 1) // enter forces the current nag
         {
-            game().addNag(NagSet::fromString(m_nagText));
+            if(!game().atGameStart())
+            {
+                game().addNag(NagSet::fromString(m_nagText));
+            }
             m_nagText.clear();
             return;
         }
-        slotGameAddVariation(m_nagText);
+        addVariation(m_nagText);
         m_nagText.clear();
         return;
     }
     else
     {
         m_nagText.append(e->text());
-        if (slotGameAddVariation(m_nagText))
+        if (addVariation(m_nagText))
         {
             m_nagText.clear();
             return;
@@ -575,14 +589,15 @@ void MainWindow::evaluateSanNag(QKeyEvent *e)
     }
 
     int matches = NagSet::prefixCount(m_nagText);
-    if(matches == 1 || enterPressed)
+    if(matches == 1)
     {
-        if(game().atGameStart())
+        if(!game().atGameStart())
         {
-            m_nagText.clear();
-            return;
+            if (!game().addNag(NagSet::fromString(m_nagText)))
+            {
+                return; // Could be start of NAG or a start of a move (R<->RR)
+            }
         }
-        game().addNag(NagSet::fromString(m_nagText));
         m_nagText.clear();
     }
 }
@@ -665,8 +680,6 @@ void MainWindow::gameLoad(int index)
             m_gameList->selectGame(index);
             emit signalGameIsEmpty(true);
             slotGameChanged();
-            emit signalFirstGameLoaded(databaseInfo()->filter()->previousGame(index) == -1);
-            emit signalLastGameLoaded(databaseInfo()->filter()->nextGame(index) == -1);
             m_gameList->setFocus();
             emit signalGameLoaded();
         }
@@ -675,7 +688,7 @@ void MainWindow::gameLoad(int index)
 
 bool MainWindow::gameMoveBy(int change)
 {
-    if(game().moveByPly(change))
+    if(!gameMode() && game().moveByPly(change))
     {
         m_gameView->setFocus();
         return true;
@@ -742,6 +755,31 @@ void MainWindow::setFavoriteDatabase(QString fname)
     }
 }
 
+void MainWindow::slotGameMoveFirst()
+{
+    gameMoveBy(-999);
+}
+
+void MainWindow::slotGameMovePrevious()
+{
+    gameMoveBy(-1);
+}
+
+void MainWindow::slotGameMoveLast()
+{
+    gameMoveBy(999);
+}
+
+void MainWindow::slotGameMoveNextN()
+{
+    gameMoveBy(10);
+}
+
+void MainWindow::slotGameMovePreviousN()
+{
+    gameMoveBy(-10);
+}
+
 void MainWindow::openDatabase(QString fname)
 {
     openDatabaseUrl(fname, false);
@@ -1205,17 +1243,18 @@ void MainWindow::setupActions()
     QMenu* loadMenu = gameMenu->addMenu(tr("&Load"));
 
     /* Game->Load submenu */
-    QAction * nextAction = createAction(tr("&Next"), SLOT(slotGameLoadNext()), Qt::CTRL + Qt::SHIFT + Qt::Key_Down,
-                                        dbToolBar, ":/images/game_down.png");
-    connect(this, SIGNAL(signalLastGameLoaded(bool)), nextAction, SLOT(setDisabled(bool)));
-    loadMenu->addAction(nextAction);
-    QAction * prevAction = createAction(tr("&Previous"), SLOT(slotGameLoadPrevious()), Qt::CTRL + Qt::SHIFT + Qt::Key_Up,
+    QAction * prevAction = createAction(tr("&Previous"), SLOT(slotGameLoadPrevious()), Qt::Key_F3,
                                         dbToolBar, ":/images/game_up.png");
-    connect(this, SIGNAL(signalFirstGameLoaded(bool)), prevAction, SLOT(setDisabled(bool)));
+    QAction * nextAction = createAction(tr("&Next"), SLOT(slotGameLoadNext()), Qt::Key_F4,
+                                        dbToolBar, ":/images/game_down.png");
+    connect(m_gameList, SIGNAL(signalLastGameLoaded(bool)), nextAction, SLOT(setDisabled(bool)));
+    connect(m_gameList, SIGNAL(signalFirstGameLoaded(bool)), prevAction, SLOT(setDisabled(bool)));
+    loadMenu->addAction(nextAction);
     loadMenu->addAction(prevAction);
     loadMenu->addAction(createAction(tr("&Go to game..."), SLOT(slotGameLoadChosen()), Qt::CTRL + Qt::Key_G));
     loadMenu->addAction(createAction(tr("&Random"), SLOT(slotGameLoadRandom()), Qt::CTRL + Qt::Key_Question));
-    gameMenu->addAction(createAction(tr("&Save...."), SLOT(slotGameSave()), QKeySequence::Save));
+    gameMenu->addAction(createAction(tr("&Save..."), SLOT(slotGameSave()), QKeySequence::Save));
+    gameMenu->addAction(createAction(tr("Edit tags..."), SLOT(slotGameEditTags())));
 
     gameMenu->addSeparator();
 
@@ -1226,6 +1265,14 @@ void MainWindow::setupActions()
     ExclusiveActionGroup* autoGroup = new ExclusiveActionGroup(this);
     ExclusiveActionGroup* autoGroup2 = new ExclusiveActionGroup(this);
 
+    m_match = createAction(tr("Match"), SLOT(slotToggleGameMode()), Qt::CTRL + Qt::Key_M, gameToolBar, ":/images/black_chess.png");
+    m_match->setCheckable(true);
+    autoGroup2->addAction(m_match);
+    autoGroup->addAction(m_match);
+    gameMenu->addAction(m_match );
+    gameMenu->addSeparator();
+    connect(m_match, SIGNAL(changed()), SLOT(slotToggleGameMode()));
+
     m_training = createAction(tr("Training"), SLOT(slotToggleTraining()), Qt::CTRL + Qt::Key_R, gameToolBar, ":/images/training.png");
     m_training->setCheckable(true);
     autoGroup2->addAction(m_training);
@@ -1329,8 +1376,9 @@ void MainWindow::setupActions()
     /* Database menu */
     QMenu* menuDatabase = menuBar()->addMenu(tr("&Database"));
     m_menuDatabases = menuDatabase->addMenu(tr("&Switch to"));
-    menuDatabase->addAction(createAction(tr("&Copy games..."), SLOT(slotDatabaseCopy()),
-                                         Qt::Key_F5));
+    menuDatabase->addAction(createAction(tr("&Copy games..."), SLOT(slotDatabaseCopy()), Qt::Key_F5));
+    menuDatabase->addSeparator();
+    menuDatabase->addAction(createAction(tr("Clear clipboard"), SLOT(slotDatabaseClearClipboard())));
 
     /* Help menu */
     menuBar()->addSeparator();
@@ -1448,7 +1496,7 @@ bool MainWindow::QuerySaveGame(DatabaseInfo *dbInfo)
     }
     if(dbInfo->gameNeedsSaving())
     {
-        SaveDialog dlg;
+        SaveDialog dlg(this);
         int n = dlg.save(dbInfo->database(), dbInfo->currentGame());
         if(n == QDialog::Accepted)
         {
@@ -1533,7 +1581,7 @@ void MainWindow::StartCheckDatabase()
 {
     if ((m_recentFiles.count() == 0) && !AppSettings->getValue("/General/BuiltinDbInstalled").toBool())
     {
-        LoadQuery dlg;
+        LoadQuery dlg(this);
         if(dlg.exec() == QDialog::Accepted)
         {
             slotStatusMessage(tr("Start loading database..."));
@@ -1563,7 +1611,9 @@ void MainWindow::slotVersionFound(int major, int minor, int build)
 void MainWindow::dragEnterEvent(QDragEnterEvent *event)
 {
     const QMimeData *mimeData = event->mimeData();
-    if(mimeData->hasUrls())
+    const GameMimeData* gameMimeData = qobject_cast<const GameMimeData*>(mimeData);
+    const DbMimeData* dbMimeData = qobject_cast<const DbMimeData*>(mimeData);
+    if(mimeData->hasUrls() || gameMimeData || dbMimeData)
     {
         event->acceptProposedAction();
     }
@@ -1621,3 +1671,13 @@ void MainWindow::slotAutoSwitchTab()
         m_tabDragIndex = -1;
     }
 }
+
+bool MainWindow::gameMode() const
+{
+    return m_gameMode;
+}
+
+void MainWindow::setGameMode(bool gameMode)
+{
+    m_gameMode = gameMode;
+}
