@@ -12,8 +12,6 @@
 
 #include "common.h"
 #include "historylist.h"
-#include "kbaction.h"
-#include "move.h"
 #include "output.h"
 
 #include <QtGui>
@@ -34,6 +32,7 @@ class ECOListWidget;
 class EcoThread;
 class EditAction;
 class EventListWidget;
+class FicsClient;
 class Game;
 class GameList;
 class OpeningTreeWidget;
@@ -103,7 +102,16 @@ protected:
     bool gameMoveBy(int change);
     /** Update recent files menu */
     void updateMenuRecent();
+    /** Get current mode */
+    bool gameMode() const;
+    /** Set current Mode */
+    void setGameMode(bool gameMode);
+    /** Add a variation from keyboard */
+    bool addVariation(const QString& s);
 public slots:
+    /** Enter gaming mode */
+    void slotToggleGameMode();
+    void enterGameMode(bool gameMode);
     /** Open database */
     void openDatabase(QString fname);
     /** Open database from URL*/
@@ -152,32 +160,17 @@ public slots:
     void slotHelpBug();
     void slotConfigureFlip();
     /** Go to first move of the game */
-    void slotGameMoveFirst()
-    {
-        gameMoveBy(-999);
-    }
+    void slotGameMoveFirst();
     /** Go to next move of the game */
     bool slotGameMoveNext();
     /** Go to previous move of the game */
-    void slotGameMovePrevious()
-    {
-        gameMoveBy(-1);
-    }
+    void slotGameMovePrevious();
     /** Go to last move of the game */
-    void slotGameMoveLast()
-    {
-        gameMoveBy(999);
-    }
+    void slotGameMoveLast();
     /** Go N moves forward in the game. For now fixed at @p 10 */
-    void slotGameMoveNextN()
-    {
-        gameMoveBy(10);
-    }
+    void slotGameMoveNextN();
     /** Go N moves backward in the game. For now fixed at @p 10 */
-    void slotGameMovePreviousN()
-    {
-        gameMoveBy(-10);
-    }
+    void slotGameMovePreviousN();
     /** Load first game */
     void slotGameLoadFirst();
     /** Load next game */
@@ -203,6 +196,7 @@ public slots:
     /** Get a pointer or Null to the active game. */
     void slotGetActiveGame(const Game** game);
     /** Merge the game with index @p gameIndex into the active game. */
+    void slotMergeActiveGame(QList<int> gameIndexList);
     void slotMergeActiveGame(int gameIndex);
     /** Merge all games from the database into the active game. */
     void slotMergeAllGames();
@@ -223,6 +217,8 @@ public slots:
     /** Save game, replacing old one if it is edited, appending if it is new
         @return false if the saving was cancelled, true if changes are to be saved or discarded. */
     bool slotGameSave();
+    /** Start a dialog which edits the games tags */
+    void slotGameEditTags();
     /** Add variation to current position. */
     void slotGameAddVariation(const Analysis& analysis);
     /** Add variation to current position. */
@@ -310,13 +306,15 @@ public slots:
     /** Change database. */
     void slotDatabaseChange();
     /** Copy games between databases. */
-    void slotDatabaseCopy(int preselect = 1, int index = -1);
+    void slotDatabaseCopy(int preselect = 1, QList<int> gameIndexList = QList<int>());
+    /** Clear the clipboard database */
+    void slotDatabaseClearClipboard();
     /** Copy games between databases. */
-    void slotDatabaseCopySingle(int n);
+    void slotDatabaseCopySingle(QList<int> listGames);
     /** Database was changed - change informations. */
     void slotDatabaseChanged();
     /** Delete current game. */
-    void slotDatabaseDeleteGame(int n);
+    void slotDatabaseDeleteGame(QList<int> gameIndexList);
     /** Delete all game in filter. */
     void slotDatabaseDeleteFilter();
     /** Slot that updates internal info upon loading a complete db */
@@ -391,7 +389,7 @@ protected slots:
 
 protected:
     void moveChanged();
-    bool pasteFen(QString& errorText, QString fen);
+    bool pasteFen(QString& errorText, QString fen, bool newGame=false);
     void copyGame(int target, int index);
     Database* getDatabaseByPath(QString path);
     DatabaseInfo* getDatabaseInfoByPath(QString path);
@@ -431,8 +429,6 @@ signals:
     void signalCurrentDBcanBeClosed(bool);
     void signalCurrentDBhasGames(bool);
 
-    void signalFirstGameLoaded(bool);
-    void signalLastGameLoaded(bool);
     void signalGameLoaded();
 
     void signalVersionFound(int, int, int);
@@ -542,6 +538,7 @@ private:
     Board m_AutoInsertLastBoard;
     Square m_annotationSquare;
     Square m_annotationSquareFrom;
+    QAction* m_match;
     QAction* m_training;
     QAction* m_autoRespond;
     QAction* m_autoPlay;
@@ -551,6 +548,8 @@ private:
     DownloadManager* downloadManager;
     bool m_machineHasToMove;
     bool m_bInDrag;
+    bool m_gameMode;
+    FicsClient* m_ficsClient;
 };
 
 
