@@ -18,11 +18,11 @@
 #include "compileeco.h"
 #include "databaseinfo.h"
 #include "databaselist.h"
-#include "ecolistwidget.h"
 #include "dlgsavebook.h"
+#include "ecolistwidget.h"
 #include "editaction.h"
 #include "eventlistwidget.h"
-#include "filtermodel.h"
+#include "ficsclient.h"
 #include "game.h"
 #include "gamelist.h"
 #include "mainwindow.h"
@@ -36,11 +36,11 @@
 #include "preferences.h"
 #include "promotiondialog.h"
 #include "renametagdialog.h"
-#include "savedialog.h"
 #include "shellhelper.h"
 #include "settings.h"
 #include "tablebase.h"
 #include "tableview.h"
+#include "tagdialog.h"
 #include "version.h"
 
 #include <time.h>
@@ -206,11 +206,14 @@ void MainWindow::slotFileClose()
                     }
                 }
 
-                m_currentDatabase = 0; // Switch to clipboard is always safe
-                m_boardView->setDbIndex(m_currentDatabase);
-                UpdateBoardInformation();
-                m_databaseList->setFileCurrent(QString());
-                slotDatabaseChanged();
+                if (m_currentDatabase != 0)
+                {
+                    m_currentDatabase = 0; // Switch to clipboard is always safe
+                    m_boardView->setDbIndex(m_currentDatabase);
+                    UpdateBoardInformation();
+                    m_databaseList->setFileCurrent(QString());
+                    slotDatabaseChanged();
+                }
             }
         }
     }
@@ -424,7 +427,7 @@ void MainWindow::slotEditVarRemove()
     }
 }
 
-bool MainWindow::pasteFen(QString& msg, QString fen)
+bool MainWindow::pasteFen(QString& msg, QString fen, bool newGame)
 {
     // Prepare Fen - clean up code like this:
     // [FEN "***"] to ***
@@ -457,7 +460,11 @@ bool MainWindow::pasteFen(QString& msg, QString fen)
                  "You can only paste such positions in <b>Setup position</b> dialog.");
         return false ;
     }
-    game().setStartingBoard(board);
+    if (newGame)
+    {
+        slotGameNew();
+    }
+    game().setStartingBoard(board,"");
     return true;
 }
 
@@ -483,8 +490,7 @@ bool MainWindow::slotEditPastePGN()
             if(pgnDatabase.loadGame(0, g))
             {
                 slotGameNew();
-                game() = g;
-                slotGameChanged();
+                game().copyFromGame(g);
                 return true;
             }
         }
@@ -496,7 +502,7 @@ void MainWindow::slotEditPaste()
 {
     QString fen = QApplication::clipboard()->text().simplified();
     QString dummy;
-    if(!pasteFen(dummy, fen))
+    if(!pasteFen(dummy, fen, true))
     {
         slotEditPastePGN();
     }
@@ -545,7 +551,7 @@ void MainWindow::slotEditBoard()
     dlg.setFlipped(m_boardView->isFlipped());
     if(dlg.exec() == QDialog::Accepted)
     {
-        game().setStartingBoard(dlg.board());
+        game().setStartingBoard(dlg.board(),tr("Set starting board"));
     }
 }
 
@@ -753,13 +759,13 @@ void MainWindow::moveChanged()
     // Clear  entries
     m_nagText.clear();
 
-    emit signalMoveHasNextMove(!game().atLineEnd());
-    emit signalMoveHasPreviousMove(!game().atGameStart());
-    emit signalMoveHasVariation(game().variationCount() > 0);
-    emit signalMoveHasParent(!game().isMainline());
-    emit signalVariationHasSibling(game().variationHasSiblings(m));
+    emit signalMoveHasNextMove(!gameMode() && !game().atLineEnd());
+    emit signalMoveHasPreviousMove(!gameMode() && !game().atGameStart());
+    emit signalMoveHasVariation(!gameMode() && game().variationCount() > 0);
+    emit signalMoveHasParent(!gameMode() && !game().isMainline());
+    emit signalVariationHasSibling(!gameMode() && game().variationHasSiblings(m));
     emit signalGameIsEmpty(false);
-    emit signalGameAtLineStart(game().atLineStart());
+    emit signalGameAtLineStart(!gameMode() && game().atLineStart());
 }
 
 void MainWindow::slotSearchTree()
@@ -885,18 +891,12 @@ void MainWindow::slotGameLoadLast()
 
 void MainWindow::slotGameLoadPrevious()
 {
-    int index = m_gameList->currentIndex().row();
-    int game = databaseInfo()->filter()->indexToGame(index);
-    game = databaseInfo()->filter()->previousGame(game);
-    gameLoad(game);
+    m_gameList->selectPreviousGame();
 }
 
 void MainWindow::loadNextGame()
 {
-    int index = m_gameList->currentIndex().row();
-    int game = databaseInfo()->filter()->indexToGame(index);
-    game = databaseInfo()->filter()->nextGame(game);
-    gameLoad(game);
+    m_gameList->selectNextGame();
 }
 
 void MainWindow::slotGameLoadNext()
@@ -931,9 +931,7 @@ void MainWindow::slotGameNew()
         if(QuerySaveGame())
         {
             databaseInfo()->newGame();
-            m_gameList->clearSelection();
-            emit signalFirstGameLoaded(true);
-            emit signalLastGameLoaded(true);
+            m_gameList->removeSelection();
             emit signalGameIsEmpty(true); // repair effect of slotGameChanged
             m_gameList->setFocus();
         }
@@ -945,7 +943,7 @@ void MainWindow::saveGame(DatabaseInfo* dbInfo)
     if(!dbInfo->database()->isReadOnly())
     {
         dbInfo->saveGame();
-        dbInfo->database()->index()->setTag("Length", QString::number((game().plyCount() + 1) / 2), gameIndex());
+        dbInfo->database()->index()->setTag("Length", QString::number((dbInfo->currentGame().plyCount() + 1) / 2), dbInfo->currentIndex());
         m_gameList->updateFilter();
         slotFilterChanged();
         slotGameChanged();
@@ -978,6 +976,15 @@ bool MainWindow::slotGameSave()
     return QuerySaveGame();
 }
 
+void MainWindow::slotGameEditTags()
+{
+    TagDialog dlg(this);
+    if (dlg.editTags(database()->index(), game(),databaseInfo()->currentIndex()))
+    {
+        databaseInfo()->saveGame();
+    }
+}
+
 void MainWindow::slotGameModify(const EditAction& action)
 {
     if((action.type() != EditAction::CopyHtml) &&
@@ -1060,6 +1067,26 @@ void MainWindow::slotGameModify(const EditAction& action)
     }
 }
 
+void MainWindow::slotMergeActiveGame(QList<int> gameIndexList)
+{
+    if (gameIndexList.size())
+    {
+        Game state = game();
+        foreach(int gameIndex, gameIndexList)
+        {
+            if(gameIndex != databaseInfo()->currentIndex())
+            {
+                Game g;
+                if(database()->loadGame(gameIndex, g))
+                {
+                    game().dbMergeWithGame(g);
+                }
+            }
+        }
+        databaseInfo()->setModified(true,state,tr("Merge selected games"));
+    }
+}
+
 void MainWindow::slotMergeActiveGame(int gameIndex)
 {
     Game g;
@@ -1114,6 +1141,10 @@ void MainWindow::slotGameChanged()
 
 void MainWindow::slotGameViewLink(const QUrl& url)
 {
+    if (gameMode())
+    {
+        return;
+    }
     if(url.scheme() == "move")
     {
         if(url.path() == "prev")
@@ -1199,12 +1230,9 @@ void MainWindow::slotGameAddVariation(const Analysis& analysis)
     game().addVariation(analysis.variation(), score);
 }
 
-bool MainWindow::slotGameAddVariation(const QString& san)
+bool MainWindow::addVariation(const QString& s)
 {
     bool added = false;
-    QString s = san;
-    s = s.remove(QRegExp("-.*"));
-    s = s.remove(QRegExp("[0-9]*\\."));
     if(game().atLineEnd())
     {
         added = game().addMove(s) != NO_MOVE;
@@ -1216,6 +1244,14 @@ bool MainWindow::slotGameAddVariation(const QString& san)
     return added;
 }
 
+bool MainWindow::slotGameAddVariation(const QString& san)
+{
+    QString s = san;
+    s = s.remove(QRegExp("-.*"));
+    s = s.remove(QRegExp("[0-9]*\\."));   
+    return addVariation(s);
+}
+
 void MainWindow::slotGameUncomment()
 {
     game().removeComments();
@@ -1507,7 +1543,7 @@ void MainWindow::copyDatabase(QString target, QString src)
     }
 }
 
-void MainWindow::slotDatabaseCopy(int preselect, int index)
+void MainWindow::slotDatabaseCopy(int preselect, QList<int> gameIndexList)
 {
     if(m_databases.count() < 2)
     {
@@ -1516,6 +1552,11 @@ void MainWindow::slotDatabaseCopy(int preselect, int index)
     }
     Game g;
     QString players;
+    int index = -1;
+    if (gameIndexList.size())
+    {
+        index = gameIndexList.at(0);
+    }
     if (index == -1)
     {
         players = game().tag(TagNameWhite)+"-"+game().tag(TagNameBlack);
@@ -1559,6 +1600,15 @@ void MainWindow::slotDatabaseCopy(int preselect, int index)
             m_databases[target]->database()->appendGame(g);
         }
         break;
+    case CopyDialog::Selection:
+        foreach (int i, gameIndexList)
+        {
+            if(database()->loadGame(i, g))
+            {
+                m_databases[target]->database()->appendGame(g);
+            }
+        }
+        break;
     case CopyDialog::Filter:
         for(int i = 0; i < (int)database()->count(); ++i)
         {
@@ -1583,9 +1633,21 @@ void MainWindow::slotDatabaseCopy(int preselect, int index)
     m_databases[target]->filter()->resize(m_databases[target]->database()->count(), true);
 }
 
-void MainWindow::slotDatabaseCopySingle(int n)
+void MainWindow::slotDatabaseClearClipboard()
 {
-    slotDatabaseCopy(0, n);
+    ((MemoryDatabase*)(m_databases[0]->database()))->clear();
+    m_databases[0]->filter()->resize(0, false);
+
+    if (!m_currentDatabase)
+    {
+        emit databaseChanged(databaseInfo());
+        emit databaseModified();
+    }
+}
+
+void MainWindow::slotDatabaseCopySingle(QList<int> gameIndexList)
+{
+    slotDatabaseCopy(0, gameIndexList);
 }
 
 void MainWindow::slotDatabaseChanged()
@@ -1607,7 +1669,7 @@ void MainWindow::slotSearchTag()
 
 void MainWindow::slotSearchBoard()
 {
-    BoardSearchDialog dlg;
+    BoardSearchDialog dlg(this);
     dlg.setBoard(m_boardView->board());
     if (dlg.exec() == QDialog::Accepted)
     {
@@ -1658,8 +1720,11 @@ void MainWindow::slotSearchReset()
 
 void MainWindow::slotTreeUpdate()
 {
-    m_gameList->updateFilter();
-    slotFilterChanged();
+    if (!gameMode())
+    {
+        m_gameList->updateFilter();
+        slotFilterChanged();
+    }
 }
 
 void MainWindow::slotSearchTreeMove(const QModelIndex& index)
@@ -1698,7 +1763,7 @@ void MainWindow::slotSearchTreeMove(const QModelIndex& index)
 
 void MainWindow::updateOpeningTree(const Board& b, bool atEnd)
 {
-    if(m_openingTreeWidget->isVisible())
+    if(!gameMode() && m_openingTreeWidget->isVisible())
     {
         QString name;
         DatabaseInfo* dbInfo;
@@ -1718,15 +1783,18 @@ void MainWindow::updateOpeningTree(const Board& b, bool atEnd)
     }
 }
 
-void MainWindow::slotDatabaseDeleteGame(int n)
+void MainWindow::slotDatabaseDeleteGame(QList<int> gameIndexList)
 {
-    if(database()->deleted(n))
+    foreach(int n, gameIndexList)
     {
-        database()->undelete(n);
-    }
-    else
-    {
-        database()->remove(n);
+        if(database()->deleted(n))
+        {
+            database()->undelete(n);
+        }
+        else
+        {
+            database()->remove(n);
+        }
     }
     m_gameList->updateFilter();
 }
@@ -2084,3 +2152,30 @@ void MainWindow::slotBookBuildError(QString /*path*/)
 {
     MessageDialog::warning(tr("Could not build book"), tr("Polyglot Error"));
 }
+
+void MainWindow::slotToggleGameMode()
+{
+    enterGameMode(m_match->isChecked());
+}
+
+void MainWindow::enterGameMode(bool gameMode)
+{
+    Guess::setGuessAllowed(!gameMode);
+    Engine::setAllowEngineOutput(!gameMode);
+    Tablebase::setAllowEngineOutput(!gameMode);
+    if (gameMode)
+    {
+        m_openingTreeWidget->cancel();
+    }
+    setGameMode(gameMode);
+#ifdef FICS_SUPPORT
+    if (gameMode)
+    {
+        m_ficsClient->startSession();
+    }
+    else
+    {
+        m_ficsClient->exitSession();
+    }
+#endif
+}
