@@ -29,15 +29,31 @@ public:
 
     QString move(QModelIndex index) const;
     Board board() const;
-    bool updateFilter(Filter& f, const Board& b, bool updateFilter, bool bEnd);
+    bool updateFilter(Filter& f, const Board& b, bool bEnd);
+    int getFilterIndex(QString& name) const;
+    void updateFilterIndex(QStringList);
+    bool shouldAddMove() const;
 
 public slots:
-    void cancel(bool bVisible);
+    void cancel();
     /** Store current configuration. */
     void saveConfig();
     /** Restore current configuration. */
     void slotReconfigure();
 
+protected slots:
+    /** Indicate end of tree update */
+    void slotTreeUpdate();
+    /** Show progress bar for open file. */
+    void slotOperationProgress(int value);
+    /** Indicate start of tree update */
+    void slotTreeUpdateStarted();
+    /** The source for the tree has changed */
+    void slotSourceChanged();
+
+signals:
+    void signalTreeUpdated();
+    void signalSourceChanged();
 
 private:
     Ui::OpeningTreeWidget *ui;
