@@ -1,5 +1,5 @@
 /****************************************************************************
-*   Copyright (C) 2010 by Michal Rudolf <mrudolf@kdewebdev.org>           *
+*   Copyright (C) 2012 by Jens Nissen jens-chessx@gmx.net                   *
 ****************************************************************************/
 
 #include "plaintextedit.h"
@@ -10,13 +10,13 @@ PlainTextEdit::PlainTextEdit(QWidget *parent) : QPlainTextEdit(parent)
 
 void PlainTextEdit::keyPressEvent(QKeyEvent* event)
 {
-    if(event->key() == Qt::Key_Return && event->modifiers() == Qt::NoModifier)
+    if ((event->key() == Qt::Key_Return) && ((event->modifiers() & Qt::KeyboardModifierMask) == Qt::NoModifier))
     {
         event->ignore();
         return;
     }
 
-    if(event->key() == Qt::Key_Return && event->modifiers() == Qt::ControlModifier)
+    if(event->key() == Qt::Key_Return && (event->modifiers() & Qt::ControlModifier))
     {
         event->ignore();
         QKeyEvent event2(QEvent::KeyPress, Qt::Key_Return, Qt::NoModifier);
