@@ -429,7 +429,7 @@ void PreferencesDialog::restoreSettings()
     AppSettings->beginGroup("/Board/");
     ui.boardFrameCheck->setChecked(AppSettings->getValue("showFrame").toBool());
     ui.boardShowCoordinates->setChecked(AppSettings->getValue("showCoordinates").toBool());
-    ui.hilightCurrentMove->setChecked(AppSettings->getValue("showCurrentMove").toBool());
+    ui.hilightCurrentMove->setCurrentIndex(AppSettings->getValue("showCurrentMove").toInt());
     ui.cbShowIndicator->setCurrentIndex(AppSettings->getValue("showMoveIndicator").toInt());
     ui.guessMoveCheck->setChecked(AppSettings->getValue("guessMove").toBool());
     ui.guessNextMove->setChecked(AppSettings->getValue("nextGuess").toBool());
@@ -518,6 +518,11 @@ void PreferencesDialog::restoreSettings()
     ui.extToolPath->setText(AppSettings->getValue("Path1").toString());
     ui.extToolParameters->setText(AppSettings->getValue("CommandLine1").toString());
     AppSettings->endGroup();
+
+    AppSettings->beginGroup("FICS");
+    ui.userName->setText(AppSettings->getValue("userName").toString());
+    ui.passWord->setText(AppSettings->getValue("passWord").toString());
+    AppSettings->endGroup();
 }
 
 void PreferencesDialog::saveSettings()
@@ -533,7 +538,7 @@ void PreferencesDialog::saveSettings()
     AppSettings->beginGroup("/Board/");
     AppSettings->setValue("showFrame", QVariant(ui.boardFrameCheck->isChecked()));
     AppSettings->setValue("showCoordinates", QVariant(ui.boardShowCoordinates->isChecked()));
-    AppSettings->setValue("showCurrentMove", QVariant(ui.hilightCurrentMove->isChecked()));
+    AppSettings->setValue("showCurrentMove", QVariant(ui.hilightCurrentMove->currentIndex()));
     AppSettings->setValue("showMoveIndicator", QVariant(ui.cbShowIndicator->currentIndex()));
     AppSettings->setValue("guessMove", QVariant(ui.guessMoveCheck->isChecked()));
     AppSettings->setValue("nextGuess", QVariant(ui.guessNextMove->isChecked()));
@@ -588,6 +593,11 @@ void PreferencesDialog::saveSettings()
     AppSettings->setValue("CommandLine1", ui.extToolParameters->text());
     AppSettings->endGroup();
 
+    AppSettings->beginGroup("FICS");
+    AppSettings->setValue("userName", ui.userName->text());
+    AppSettings->setValue("passWord", ui.passWord->text());
+    AppSettings->endGroup();
+
     QDir().mkpath(ui.defaultDataBasePath->text());
 }
 
